{********************************************************************}
{                                                                    }
{ written by TMS Software                                            }
{            copyright (c) 2020                                      }
{            Email : info@tmssoftware.com                            }
{            Web : http://www.tmssoftware.com                        }
{                                                                    }
{ The source code is given as is. The author is not responsible      }
{ for any possible damage done due to the use of this code.          }
{ The complete source code remains property of the author and may    }
{ not be distributed, published, given or sold in any form as such.  }
{ No parts of the source code can be included in any other component }
{ or application without written authorization of the author.        }
{********************************************************************}

unit WEBLib.Actions;

interface

uses
  Classes, SysUtils, Web, JS, Controls, WEBLib.Controls;

type
  THTMLEvent = (heClick,heDblClick,heKeypress,heKeydown,heKeyup,heMouseDown,heMouseMove,heMouseUp,heMouseEnter,heMouseLeave,heBlur,heFocus,heChange,heSelect,heInput,heInvalid,heCustom,heNone);

  THTMLAction = (actNone, actSetHidden, actRemoveHidden, actToggleHidden, actSetReadOnly, actRemoveReadOnly, actToggleReadOnly, actSetDisabled, actRemoveDisabled, actToggleDisabled, actClear);

  TJSEventParameter = record
    JSEvent: TJSEvent;
  end;

  THTMLEventEvent = procedure(Sender: TObject; Element: TJSHTMLElementRecord; Event: TJSEventParameter) of object;

  THTMLUpdateEvent = procedure(Sender: TObject; Element: TJSHTMLElementRecord; Event: TJSEventParameter; TargetElement: TJSHTMLElementRecord) of object;

  TCustomHTMLEvent = type string;

  TElementArray = array of TJSElement;

  TElementAction = class(TCollectionItem)
  private
    FElementEvent: THTMLEvent;
    FElementID: TElementID;
    FOnExecute: THTMLEventEvent;
    FTag: integer;
    FStopPropagation: boolean;
    FPreventDefault: boolean;
    FCustomEvent: TCustomHTMLEvent;
    FSelector: string;
    FEnabled: boolean;
    FControl: TControl;
    FOnUpdate: THTMLUpdateEvent;
    FTargetControl: TControl;
    FTargetID: TElementID;
    FTargetAction: THTMLAction;
    FTargetSelector: string;
    FName: string;
    FElementHandle: TJSElement;
    FElementHandles: TElementArray;
    procedure SetSelector(const Value: string);
    procedure SetElementID(const Value: TElementID);
    procedure SetCustomEvent(const Value: TCustomHTMLEvent);
    procedure SetName(const Value: string);
  protected
    function GetDisplayName: string; override;
  public
    procedure Assign(Source: TPersistent); override;
    constructor Create(Collection: TCollection); override;
    procedure Bind;
    procedure UnBind;
    property ElementHandle: TJSElement read FElementHandle;
    property ElementHandles: TElementArray read FElementHandles;
  published
    property Control: TControl read FControl write FControl;
    property CustomEvent: TCustomHTMLEvent read FCustomEvent write SetCustomEvent;
    property Enabled: boolean read FEnabled write FEnabled default true;
    property Event: THTMLEvent read FElementEvent write FElementEvent default heClick;
    property ID: TElementID read FElementID write SetElementID;
    property Name: string read FName write SetName;
    property PreventDefault: boolean read FPreventDefault write FPreventDefault default true;
    property Selector: string read FSelector write SetSelector;
    property StopPropagation: boolean read FStopPropagation write FStopPropagation default true;
    property Tag: integer read FTag write FTag default 0;
    property TargetAction: THTMLAction read FTargetAction write FTargetAction default actNone;
    property TargetControl: TControl read FTargetControl write  FTargetControl;
    property TargetID: TElementID read FTargetID write FTargetID;
    property TargetSelector: string read FTargetSelector write FTargetSelector;
    property OnExecute: THTMLEventEvent read FOnExecute write FOnExecute;
    property OnUpdate: THTMLUpdateEvent read FOnUpdate write FOnUpdate;
  end;

  TElementActions = class(TOwnedCollection)
  private
    function GetItems(AIndex: integer): TElementAction;
    procedure SetItems(AIndex: integer; const Value: TElementAction);
    function GetAction(AName: string): TElementAction;
  protected
  public
    constructor Create(AOwner: TComponent);
    function Add: TElementAction; reintroduce;
    function Insert(AIndex: integer): TElementAction; reintroduce;
    property Items[AIndex: integer]: TElementAction read GetItems write SetItems; default;
    function GetActionByName(AName: string): TElementAction;
    property Action[AName: string]: TElementAction Read GetAction;
    function GetByName(AName: string): TElementAction;
    function FindByName(AName: string): TElementAction;
  end;

  THTMLActionEventEvent = procedure(Sender: TObject; AAction: TElementAction; Element: TJSHTMLElementRecord; Event: TJSEventParameter) of object;

  THTMLActionUpdateEvent = procedure(Sender: TObject; AAction: TElementAction; Element: TJSHTMLElementRecord; Event: TJSEventParameter; TargetElement: TJSHTMLElementRecord) of object;


  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TElementActionList = class(TComponent)
  private
    FActions: TElementActions;
    FOnUpdate: THTMLActionUpdateEvent;
    FOnExecute: THTMLActionEventEvent;
    procedure SetActions(const Value: TElementActions);
    function GetAction(AName: string): TElementAction;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Assign(Source: TPersistent); override;
    procedure BindActions;
    procedure UnBindActions;
    function GetActionByName(AName: string): TElementAction;
    property Action[AName: string]: TElementAction Read GetAction; default;
    function GetByName(AName: string): TElementAction;
    function FindByName(AName: string): TElementAction;
  published
    property Actions: TElementActions read FActions write SetActions;
    property OnExecute: THTMLActionEventEvent read FOnExecute write FOnExecute;
    property OnUpdate: THTMLActionUpdateEvent read FOnUpdate write FOnUpdate;
  end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebElementActionList = class(TElementActionList);

implementation

{ TElementAction }

function TElementActions.Add: TElementAction;
begin
  Result := TElementAction(inherited Add);
end;

constructor TElementActions.Create(AOwner: TComponent);
begin
  inherited Create(AOwner, TElementAction);
end;

function TElementActions.FindByName(AName: string): TElementAction;
begin
  Result := nil;
end;

function TElementActions.GetAction(AName: string): TElementAction;
begin
  Result := nil;
end;

function TElementActions.GetActionByName(AName: string): TElementAction;
begin
  Result := nil;
end;

function TElementActions.GetByName(AName: string): TElementAction;
begin
  Result := nil;
end;

function TElementActions.GetItems(AIndex: integer): TElementAction;
begin
  Result := TElementAction(inherited Items[AIndex]);
end;

function TElementActions.Insert(AIndex: integer): TElementAction;
begin
  Result := TElementAction(inherited Insert(AIndex));
end;

procedure TElementActions.SetItems(AIndex: integer;
  const Value: TElementAction);
begin
  inherited Items[AIndex] := Value;
end;

{ TElementAction }

procedure TElementAction.Assign(Source: TPersistent);
begin
  if (Source is TElementAction) then
  begin
    FControl := (Source as TElementAction).Control;
    FCustomEvent := (Source as TElementAction).CustomEvent;
    FEnabled := (Source as TElementAction).Enabled;
    FElementEvent := (Source as TElementAction).Event;
    FElementID := (Source as TElementAction).ID;
    FName := (Source as TElementAction).Name;
    FPreventDefault := (Source as TElementAction).PreventDefault;
    FSelector := (Source as TElementAction).Selector;
    FStopPropagation := (Source as TElementAction).StopPropagation;
    FTag := (Source as TElementAction).Tag;
    FTargetAction := (Source as TElementAction).TargetAction;
    FTargetControl := (Source as TElementAction).TargetControl;
    FTargetID := (Source as TElementAction).TargetID;
    FTargetSelector := (Source as TElementAction).TargetSelector;
    FOnExecute := (Source as TElementAction).OnExecute;
    FOnUpdate := (Source as TElementAction).OnUpdate;
  end;
end;

procedure TElementAction.Bind;
begin
  //
end;

constructor TElementAction.Create(Collection: TCollection);
begin
  inherited;
  FElementEvent := heClick;
  FTag := 0;
  FStopPropagation := true;
  FPreventDefault := true;
  FEnabled := true;
  FControl := nil;
  FName := 'Action'+IntToStr(Index);
end;

function TElementAction.GetDisplayName: string;
var
  s: string;
begin
  s := FName;

  if (s = '') then
  begin
    case Event of
      heClick: s := 'click';
      heDblClick: s := 'dblclick';
      heKeypress:  s := 'keypress';
      heKeydown:  s := 'keydown';
      heKeyup:  s := 'keyup';
      heMouseDown:  s := 'mousedown';
      heMouseMove:  s := 'mousemove';
      heMouseUp: s := 'mousemove';
      heMouseEnter:  s := 'mouseenter';
      heMouseLeave:  s := 'mouseleave';
      heBlur:  s := 'blur';
      heFocus:  s := 'focus';
      heChange:  s := 'change';
      heInvalid:  s := 'invalid';
      heInput:  s := 'input';
    end;

    if ID <> '' then
      Result := s +'event_' + ID
    else
      Result := s +'event_' + IntToStr(Index);
  end
  else
    Result := s;
end;

procedure TElementAction.SetCustomEvent(const Value: TCustomHTMLEvent);
begin
  if (FCustomEvent <> Value) then
  begin
    FCustomEvent := Value;
    Event := heCustom;
  end;
end;

procedure TElementAction.SetElementID(const Value: TElementID);
begin
  if (FElementID <> Value) then
  begin
    FElementID := Value;
    if FElementID <> '' then
      FSelector := '';
  end;
end;

procedure TElementAction.SetName(const Value: string);
begin
  FName := Value;
end;

procedure TElementAction.SetSelector(const Value: string);
begin
  if (FSelector <> Value) then
  begin
    FSelector := Value;
    if FSelector <> '' then
      FElementID := '';
  end;
end;

procedure TElementAction.UnBind;
begin
  //
end;

{ TElementActionList }

procedure TElementActionList.Assign(Source: TPersistent);
begin
  if (Source is TElementActionList) then
  begin
    FActions.Assign((Source as TElementActionList).Actions);
  end;
end;

procedure TElementActionList.BindActions;
begin
end;

constructor TElementActionList.Create(AOwner: TComponent);
begin
  inherited;
  FActions := TElementActions.Create(Self);
end;

destructor TElementActionList.Destroy;
begin
  FActions.Free;
  inherited;
end;

function TElementActionList.FindByName(AName: string): TElementAction;
begin
  Result := nil;
end;

function TElementActionList.GetAction(AName: string): TElementAction;
begin
  Result := Actions.GetActionByName(AName);
end;

function TElementActionList.GetActionByName(AName: string): TElementAction;
begin
  Result := Actions.GetActionByName(AName);
end;

function TElementActionList.GetByName(AName: string): TElementAction;
begin
  Result := nil;
end;

procedure TElementActionList.SetActions(const Value: TElementActions);
begin
  FActions.Assign(Value);
end;

procedure TElementActionList.UnBindActions;
begin
  //
end;

end.
