{********************************************************************}
{                                                                    }
{ written by TMS Software                                            }
{            Copyright (c) 2018 - 2020                               }
{            Email : info@tmssoftware.com                            }
{            Web : http://www.tmssoftware.com                        }
{                                                                    }
{ The source code is given as is. The author is not responsible      }
{ for any possible damage done due to the use of this code.          }
{ The complete source code remains property of the author and may    }
{ not be distributed, published, given or sold in any form as such.  }
{ No parts of the source code can be included in any other component }
{ or application without written authorization of the author.        }
{********************************************************************}

unit  WEBLib.CDS;

interface

uses
  {$IFNDEF LCLTMSWEB}
  System.Classes, DB, DBClient, WEBLib.Controls, Web, JS, jsdelphisystem;
  {$ENDIF}
  {$IFDEF LCLTMSWEB}
  Classes, DB, WEBLib.Controls, Web, JS, jsdelphisystem, bufdataset;
  {$ENDIF}

type
  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebClientDataSource = class(TDataSource)
  end;

  TConnectErrorEvent = procedure(Sender: TObject; ErrorCode: integer) of object;
  TDataReceivedEvent = procedure(Sender: TObject; ARequest: TJSXMLHttpRequestRecord; var AResponse: string) of object;


  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebClientConnection = class(TComponent)
  private
    FActive: boolean;
    FDataNode: string;
    FURI: string;
    FOnConnectError: TConnectErrorEvent;
    FBeforeConnect: TNotifyEvent;
    FAfterConnect: TNotifyEvent;
    FAutoOpenDataSet: boolean;
    FHeaders: TStringList;
    FPassword: string;
    FUser: string;
    FOnDataReceived: TDataReceivedEvent;
    procedure SetHeaders(const Value: TStringList);
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property Active: boolean read FActive write FActive;
    property AutoOpenDataSet: boolean read FAutoOpenDataSet write FAutoOpenDataSet default true;
    property Headers: TStringList read FHeaders write SetHeaders;
    property DataNode: string read FDataNode write FDataNode;
    property Password: string read FPassword write FPassword;
    property User: string read FUser write FUser;

    property URI: string read FURI write FURI;
    property AfterConnect: TNotifyEvent read FAfterConnect write FAfterConnect;
    property BeforeConnect: TNotifyEvent read FBeforeConnect write FBeforeConnect;
    property OnConnectError: TConnectErrorEvent read FOnConnectError write FOnConnectError;
    property OnDataReceived: TDataReceivedEvent read FOnDataReceived write FOnDataReceived;
  end;

  TDataProxy = class(TComponent)
  end;

  TRecordUpdateDescriptor = class(TObject)
  private
    FBookmark: TBookmark;
    FData: JSValue;
    FDataset: TDataset;
    FProxy: TDataProxy;
    FResolveError: String;
    FServerData: JSValue;
    FStatus: TUpdateStatus;
    FOriginalStatus : TUpdateStatus;
  protected
  public
    property Proxy : TDataProxy read FProxy;
    property Dataset : TDataset Read FDataset;
    property OriginalStatus : TUpdateStatus Read FOriginalStatus;
    property Status : TUpdateStatus Read FStatus;
    property ServerData : JSValue Read FServerData;
    property Data : JSValue Read FData;
    property Bookmark : TBookmark Read FBookmark;
    property ResolveError : String Read FResolveError;
  end;

  TRecordUpdateDescriptorClass = class of TRecordUpdateDescriptor;

  TUpdateRecordEvent = procedure(Sender: TObject; ADescriptor: TRecordUpdateDescriptor) of object;


  TIndexOption = (ixPrimary, ixUnique, ixDescending, ixCaseInsensitive,
    ixExpression, ixNonMaintained);
  TIndexOptions = set of TIndexOption;

  TNamedItem = class(TCollectionItem)
  private
    FName: string;
    FDisplayName: string;
  protected
  Public
    property DisplayName : string read FDisplayName write FDisplayName;
  published
    property Name : string read FName write FName;
  end;

  TIndexDefs = class;

  { TIndexDef }

  TIndexDef = class(TNamedItem)
  Private
    FCaseinsFields: string;
    FDescFields: string;
    FExpression : String;
    FFields : String;
    FOptions : TIndexOptions;
    FSource : String;
  public
    constructor Create(Owner: TIndexDefs; const AName, TheFields: string;
      TheOptions: TIndexOptions); reintroduce;
    procedure Assign(Source: TPersistent); override;
  published
    property Expression: string read FExpression write FExpression;
    property Fields: string read FFields write FFields;
    property CaseInsFields: string read FCaseinsFields write FCaseInsFields;
    property DescFields: string read FDescFields write FDescFields;
    property Options: TIndexOptions read FOptions write FOptions;
    property Source: string read FSource write FSource;
  end;

  TIndexDefClass = class of TIndexDef;

  { TIndexDefs }

  TIndexDefs = class(TDefCollection)
  Private
    Function  GetItem(Index: Integer): TIndexDef; reintroduce;
    Procedure SetItem(Index: Integer; Value: TIndexDef); reintroduce;
  public
    constructor Create(ADataSet: TDataSet);
    procedure Add(const Name, Fields: string; Options: TIndexOptions); reintroduce;
    Function AddIndexDef: TIndexDef;
    function Find(const IndexName: string): TIndexDef; reintroduce;
    function FindIndexForFields(const Fields{%H-}: string): TIndexDef;
    function GetIndexForFields(const Fields: string; CaseInsensitive: Boolean): TIndexDef;
    Property Items[Index: Integer] : TIndexDef read GetItem write SetItem; default;
  end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  {$IFNDEF LCLTMSWEB}
  TWebClientDataSet = class(TDataSet)
  {$ENDIF}
  {$IFDEF LCLTMSWEB}
  TWebClientDataSet = class(TBufDataSet)
  {$ENDIF}
  private
    FConnection: TWebClientConnection;
    FOnUpdateRecord: TUpdateRecordEvent;
    FIndexes: TIndexDefs;
    FActiveIndex: string;
    FIDField: string;
    FRows: TJSArray;
    FParams: TParams;
    procedure SetParams(const Value: TParams);
  protected
    procedure InternalClose; override;
    procedure InternalHandleException; override;
    procedure InternalInitFieldDefs; override;
    procedure InternalOpen; override;
    function IsCursorOpen: Boolean; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure ApplyUpdates; reintroduce;
    property Indexes: TIndexDefs read FIndexes write FIndexes;
    property ActiveIndex: string read FActiveIndex write FActiveIndex;
    property IDField: string read FIDField write FIDField;
    procedure EmptyDataSet;
    property Rows: TJSArray read FRows write FRows;
  published
    property Active;
    property Connection: TWebClientConnection read FConnection write FConnection;
    property Params: TParams read FParams write SetParams;
    property BeforeOpen;
    property AfterOpen;
    property BeforeClose;
    property AfterClose;
    property BeforeInsert;
    property AfterInsert;
    property BeforeEdit;
    property AfterEdit;
    property BeforePost;
    property AfterPost;
    property BeforeCancel;
    property AfterCancel;
    property BeforeDelete;
    property AfterDelete;
    property BeforeScroll;
    property AfterScroll;
    property OnCalcFields;
    property OnDeleteError;
    property OnEditError;
    property OnFilterRecord;
    property OnNewRecord;
    property OnPostError;
    property OnUpdateRecord: TUpdateRecordEvent read FOnUpdateRecord write FOnUpdateRecord;
  end;

implementation


{ TWebClientConnection }


{ TWebClientDataSet }


{ TWebClientConnection }

constructor TWebClientConnection.Create(AOwner: TComponent);
begin
  inherited;
  FAutoOpenDataSet := true;
  FHeaders := TStringList.Create;
  FUser := '';
  FPassword := '';
end;

destructor TWebClientConnection.Destroy;
begin
  FHeaders.Free;
  inherited;
end;

procedure TWebClientConnection.SetHeaders(const Value: TStringList);
begin
  FHeaders.Assign(Value);
end;

{ TWebClientDataSet }

procedure TWebClientDataSet.ApplyUpdates;
begin
  //
end;

constructor TWebClientDataSet.Create(AOwner: TComponent);
begin
  inherited;
  FParams := TParams.Create(TParam)
end;

destructor TWebClientDataSet.Destroy;
begin
  FParams.Free;
  inherited;
end;

procedure TWebClientDataSet.EmptyDataSet;
begin
  //
end;

procedure TWebClientDataSet.InternalClose;
begin
  //
end;

procedure TWebClientDataSet.InternalHandleException;
begin
  //
end;

procedure TWebClientDataSet.InternalInitFieldDefs;
begin
  //
end;

procedure TWebClientDataSet.InternalOpen;
begin
  //
end;

function TWebClientDataSet.IsCursorOpen: Boolean;
begin
  Result := false;
end;

procedure TWebClientDataSet.SetParams(const Value: TParams);
begin
  FParams.Assign(Value);
end;

{ TIndexDef }

procedure TIndexDef.Assign(Source: TPersistent);
begin
  inherited;

end;

constructor TIndexDef.Create(Owner: TIndexDefs; const AName, TheFields: string;
  TheOptions: TIndexOptions);
begin
  //
  inherited Create(nil);
end;

{ TIndexDefs }

procedure TIndexDefs.Add(const Name, Fields: string; Options: TIndexOptions);
begin
  //
end;

function TIndexDefs.AddIndexDef: TIndexDef;
begin
  Result := nil;
end;

constructor TIndexDefs.Create(ADataSet: TDataSet);
begin
  //
end;

function TIndexDefs.Find(const IndexName: string): TIndexDef;
begin
  Result := nil;
end;

function TIndexDefs.FindIndexForFields(const Fields: string): TIndexDef;
begin
  Result := nil;
end;

function TIndexDefs.GetIndexForFields(const Fields: string;
  CaseInsensitive: Boolean): TIndexDef;
begin
  Result := nil;
end;

function TIndexDefs.GetItem(Index: Integer): TIndexDef;
begin
  Result := nil;
end;

procedure TIndexDefs.SetItem(Index: Integer; Value: TIndexDef);
begin
  //
end;

end.
