{********************************************************************}
{                                                                    }
{ written by TMS Software                                            }
{            Copyright (c) 2018 - 2020                               }
{            Email : info@tmssoftware.com                            }
{            Web : http://www.tmssoftware.com                        }
{                                                                    }
{ The source code is given as is. The author is not responsible      }
{ for any possible damage done due to the use of this code.          }
{ The complete source code remains property of the author and may    }
{ not be distributed, published, given or sold in any form as such.  }
{ No parts of the source code can be included in any other component }
{ or application without written authorization of the author.        }
{********************************************************************}

unit WEBLib.Controls;

interface

{$R WEBLib.CSS.RES}
{$R TMS.Web.Controls.RES}

uses
  {$IFNDEF LCLTMSWEB}
  System.Classes, Types, System.SysUtils, Rtti, Vcl.Controls, Vcl.Forms, Winapi.Messages,
  Winapi.Windows, Vcl.Graphics, Web, System.TypInfo, JS;
  {$ENDIF}
  {$IFDEF LCLTMSWEB}
  Classes, Types, SysUtils, TypInfo, LCLType, LCLIntf, Controls, Forms,
  Messages, Graphics, Web, JS;
  {$ENDIF}

const
{$IFNDEF LCLTMSWEB}
{$IF COMPILERVERSION > 28}
  pidWeb = $10000;
{$ELSE}
  pidWeb = $1000;
{$IFEND}
  TMSWebPlatform = pidWin32 + pidWin64 + pidWeb;
{$ENDIF}

  WEBDEFAULTFONT = 'Arial';

type
  TJSXMLHttpRequestRecord = record
    req: TJSXMLHttpRequest;
  end;

  TJSEventRecord = record
    event: TEventListenerEvent;
  end;

  TJSHTMLElementRecord = record
    element: TJSHTMLElement;
  end;

  TJSObjectRecord = record
    jsobject: TJSObject;
  end;

  TJSArrayRecord = record
    jsarray: TJSArray;
  end;

  TJSArrayBufferRecord = record
    jsarraybuffer: TJSArrayBuffer;
  end;

  TSizeStyle = (ssPercent, ssAbsolute, ssAuto);
  TPercentSize = 0..100;

  TElementFont = (efProperty, efCSS);
  TElementPosition = (epAbsolute, epRelative, epIgnore);

  TTextDirection = (tdDefault, tdLeftToRight, tdRightToLeft, tdInherit);

  TElementEvent = (eeClick, eeMouseDown, eeMouseUp, eeMouseMove, eeDblClick, eeKeyDown, eeKeyPress, eeKeyUp);
  TEventPropagation = set of TElementEvent;

  TTouchEvent = procedure(Sender: TObject; X,Y: integer) of object;

  TElementClassName = type string;
  TElementID = type string;
  TMaterialGlyph = type string;

  {$IFDEF LCLTMSWEB}
  TMargins = integer;
  TCustomHint = string;
  {$ENDIF}

  {$IFNDEF LCLTMSWEB}
  JSLibReferenceAttribute = class(TCustomAttribute)
  strict private
    FAttrs: string;
    FCSSs: string;
    FDesc: string;
    FSrcs: string;
  public
    constructor Create(const AScrs: string); overload;
    constructor Create(const AScrs, ACSSs: string); overload;
    constructor Create(const AScrs, ACSSs, AAttrs: string); overload;
    constructor Create(const AScrs, ACSSs, AAttrs, ADesc: string); overload;
    property Attrs: string read FAttrs;
    property CSSs: string read FCSSs;
    property Desc: string read FDesc;
    property Srcs: string read FSrcs;
  end;
  {$ENDIF}

  TWebCustomControl = class(TCustomControl)
  strict private
    FDesignTime: boolean;
    FBorderStyle: TBorderStyle;
    FElementClassName: TElementClassName;
    FElementID: TElementID;
    FChildOrder: integer;
    FRole: string;
  private
    FOnTouchMove: TTouchEvent;
    FOnTouchStart: TTouchEvent;
    FOnTouchEnd: TTouchEvent;
    FOnTouchCancel: TTouchEvent;
    FWidthStyle: TSizeStyle;
    FHeightStyle: TSizeStyle;
    FHeightPercent: TPercentSize;
    FWidthPercent: TPercentSize;
    FBorderColor: TColor;
    FElementFont: TElementFont;
    FElementPosition: TElementPosition;
    FParentElement: TJSElement;
    FEventStopPropagation: TEventPropagation;
    FParentElementID: TJSElement;
    FElementCanvas: TJSHTMLCanvasElement;
    FElementHandle: TJSHTMLElement;
    FElementEvent: TJSEvent;
    procedure CMTextChanged(var Message: TMessage); message CM_TEXTCHANGED;
    procedure SetHeightPercent(const Value: TPercentSize);
    procedure SetWidthPercent(const Value: TPercentSize);
    procedure SetHeightStyle(const Value: TSizeStyle);
    procedure SetWidthStyle(const Value: TSizeStyle);
    procedure SetBorderColor(const Value: TColor);
    function GetIsLinked: boolean;
    function GetContext: TJSCanvasRenderingContext2D;
  protected
    {$IFNDEF LCLTMSWEB}
    procedure SetParent(AParent: TWinControl); override;
    {$ENDIF}
    function GetDesignText: string; virtual;
    procedure Paint; override;
    property OnTouchStart: TTouchEvent read FOnTouchStart write FOnTouchStart;
    property OnTouchMove: TTouchEvent read FOnTouchMove write FOnTouchMove;
    property OnTouchEnd: TTouchEvent read FOnTouchEnd write FOnTouchEnd;
    property OnTouchCancel: TTouchEvent read FOnTouchCancel write FOnTouchCancel;
    property BorderColor: TColor read FBorderColor write SetBorderColor;
    property ChildOrder: integer read FChildOrder write FChildOrder default 0;
    property Role: string read FRole write FRole;
    procedure InitBootstrapStyle; virtual;
  public
    constructor Create(ID: string); reintroduce; overload; virtual;
    constructor Create(AOwner: TComponent); overload; override;
    procedure PreventDefault;
    property BorderStyle: TBorderStyle read FBorderStyle write FBorderStyle;
    procedure BeginUpdate; virtual;
    procedure EndUpdate; virtual;
    property IsLinked: boolean read GetIsLinked;
    property ElementCanvas: TJSHTMLCanvasElement read FElementCanvas;
    property ElementFont: TElementFont read FElementFont write FElementFont default efProperty;
    property ElementPosition: TElementPosition read FElementPosition write FElementPosition default epAbsolute;
    property ElementHandle: TJSHTMLElement read FElementHandle;
    property ElementEvent: TJSEvent read FElementEvent write FElementEvent;

    property EventStopPropagation: TEventPropagation read FEventStopPropagation write  FEventStopPropagation;
    property GraphicsContext: TJSCanvasRenderingContext2D read GetContext;
    property ParentElement: TJSElement read FParentElement write FParentElement;
    property ParentElementID: TJSElement read FParentElementID write FParentElementID;
  published
    property ElementClassName: TElementClassName read FElementClassName write FElementClassName;
    property ElementID: TElementID read FElementID write FElementID;
    property HeightStyle: TSizeStyle read FHeightStyle write SetHeightStyle default ssAbsolute;
    property WidthStyle: TSizeStyle read FWidthStyle write SetWidthStyle default ssAbsolute;
    property HeightPercent: TPercentSize read FHeightPercent write SetHeightPercent default 100;
    property WidthPercent: TPercentSize read FWidthPercent write SetWidthPercent default 100;
  end;

  TWebGraphicControl = class(TGraphicControl)
  private
    FWidthStyle: TSizeStyle;
    FElementID: TElementID;
    FElementFont: TElementFont;
    FParentElementID: TJSElement;
    FHeightPercent: TPercentSize;
    FElementPosition: TElementPosition;
    FHeightStyle: TSizeStyle;
    FEventStopPropagation: TEventPropagation;
    FWidthPercent: TPercentSize;
    FElementClassName: TElementClassName;
    FParentElement: TJSElement;
    FChildOrder: integer;
    FRole: string;
    function GetIsLinked: boolean;
    procedure SetHeightPercent(const Value: TPercentSize);
    procedure SetHeightStyle(const Value: TSizeStyle);
    procedure SetWidthPercent(const Value: TPercentSize);
    procedure SetWidthStyle(const Value: TSizeStyle);
  protected
    property ChildOrder: integer read FChildOrder write FChildOrder default 0;
  public
    constructor Create(AOwner: TComponent); override;
    procedure BeginUpdate; virtual;
    procedure EndUpdate; virtual;
    property IsLinked: boolean read GetIsLinked;
    property ElementFont: TElementFont read FElementFont write FElementFont default efProperty;
    property ElementPosition: TElementPosition read FElementPosition write FElementPosition default epAbsolute;
    property EventStopPropagation: TEventPropagation read FEventStopPropagation write  FEventStopPropagation;
    property ParentElement: TJSElement read FParentElement write FParentElement;
    property ParentElementID: TJSElement read FParentElementID write FParentElementID;
    property Role: string read FRole write FRole;
  published
    property ElementClassName: TElementClassName read FElementClassName write FElementClassName;
    property ElementID: TElementID read FElementID write FElementID;
    property HeightStyle: TSizeStyle read FHeightStyle write SetHeightStyle default ssAbsolute;
    property WidthStyle: TSizeStyle read FWidthStyle write SetWidthStyle default ssAbsolute;
    property HeightPercent: TPercentSize read FHeightPercent write SetHeightPercent default 100;
    property WidthPercent: TPercentSize read FWidthPercent write SetWidthPercent default 100;
  end;

  TWebNonVisualCustomControl = class(TWebCustomControl)
  protected
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); override;
    procedure SetBounds(ALeft, ATop, AWidth, AHeight: Integer); override;
  end;


  TCustomWebControl = class(TControl)
  private
    FOnTouchMove: TTouchEvent;
    FOnTouchStart: TTouchEvent;
    FOnTouchEnd: TTouchEvent;
    FParentElementID: TJSElement;
    FParentElement: TJSElement;
  public
    procedure PreventDefault;
    property ParentElement: TJSElement read FParentElement write FParentElement;
    property ParentElementID: TJSElement read FParentElementID write FParentElementID;
  protected
    property OnTouchStart: TTouchEvent read FOnTouchStart write FOnTouchStart;
    property OnTouchMove: TTouchEvent read FOnTouchMove write FOnTouchMove;
    property OnTouchEnd: TTouchEvent read FOnTouchEnd write FOnTouchEnd;
  end;

  TWebControl = class(TCustomWebControl)
  strict private
    function get_AlignWithMargins: Boolean;
    function get_Cursor: TCursor;
    function get_CustomHint: TCustomHint;
    function get_Height: Integer;
    function get_HelpContext: THelpContext;
    function get_HelpKeyword: String;
    function get_HelpType: THelpType;
    function get_Hint: string;
    function get_Left: Integer;
    function get_Margins: TMargins;
    function get_ParentCustomHint: Boolean;
    function get_Top: Integer;
    function get_Width: Integer;
  protected
    property AlignWithMargins: Boolean read get_AlignWithMargins;
  published
    property Height: Integer read get_Height;
    property Left: Integer read get_Left;
    property Top: Integer read get_Top;
    property Width: Integer read get_Width;
    property Cursor: TCursor read get_Cursor;
    property Hint: string read get_Hint;
    property HelpType: THelpType read get_HelpType;
    property HelpKeyword: String read get_HelpKeyword;
    property HelpContext: THelpContext read get_HelpContext;
    property Margins: TMargins read get_Margins;
    property CustomHint: TCustomHint read get_CustomHint;
    property ParentCustomHint: Boolean read get_ParentCustomHint;
  end;

  TWebCSSControl = class(TCustomControl)
  protected
    procedure AddControlCSS;
  published
  end;


  TCSSCodeFragment = class(TCollectionItem)
  private
    FControlClassname: string;
    FCSS: TStringList;
    procedure SetCSS(const Value: TStringList);
  public
    constructor Create(ACollection: TCollection); override;
    destructor Destroy; override;
  published
    property ControlClassname: string read FControlClassname write FControlClassname;
    property CSS: TStringList read FCSS write SetCSS;
  end;

  TCSSCodeFragments = class(TOwnedCollection)
  private
    function GetItemEx(Index: integer): TCSSCodeFragment;
    procedure SetItemEx(Index: integer; const Value: TCSSCodeFragment);
  public
    constructor Create(AOwner: TComponent); reintroduce;
    function Add: TCSSCodeFragment;
    function Insert(Index: integer): TCSSCodeFragment;
    property Items[Index: integer]: TCSSCodeFragment read GetItemEx write SetItemEx;
  end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TCSSCodeManager = class(TComponent)
  private
    FCSSFragments: TCSSCodeFragments;
    procedure SetCSSFragments(const Value: TCSSCodeFragments);
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure LoadResourceFile(AClassName, AResourceType: string; ms: TMemoryStream);
    function LoadResourceAsString(AClassName, AResourceType: string): string;
    function GetClassFragment(AClassname: string): TCSSCodeFragment;
    function GetClassCSS(AClassname: string): string;
  published
    property CSSFragments: TCSSCodeFragments read FCSSFragments write SetCSSFragments;
  end;


function IsRelativeParent(AParent: TWinControl): boolean;

procedure SetHTMLElementFont(AElement: TJSHTMLElement; AFont: TFont; UseCSS: boolean);
procedure SetHTMLElementColor(AElement: TJSHTMLElement; AColor: TColor; UseCSS: boolean);


implementation

uses
  {$IFNDEF LCLTMSWEB}
  System.UITypes,
  {$ENDIF}
  WEBLib.Forms
  ;


procedure SetHTMLElementFont(AElement: TJSHTMLElement; AFont: TFont; UseCSS: boolean);
begin
end;

procedure SetHTMLElementColor(AElement: TJSHTMLElement; AColor: TColor; UseCSS: boolean);
begin
end;

function IsRelativeParent(AParent: TWinControl): boolean;
var
  PropInfo: PPropInfo;
begin
  Result := false;

  if Assigned(AParent) then
  begin
    PropInfo := GetPropInfo(AParent.ClassInfo, 'ElementPosition');

    if Assigned(PropInfo) then
    begin
      // owner is relative
      if GetOrdProp(AParent, PropInfo) > 0 then
      begin
        Result := true;
      end;
    end;
  end;
end;

{$IFNDEF LCLTMSWEB}

{ JSLibReferenceAttribute }

constructor JSLibReferenceAttribute.Create(const AScrs: string);
begin
  inherited Create;
  FSrcs := AScrs;
end;

constructor JSLibReferenceAttribute.Create(const AScrs, ACSSs: string);
begin
  Create(AScrs);
  FCSSs := ACSSs;
end;

constructor JSLibReferenceAttribute.Create(const AScrs, ACSSs, AAttrs: string);
begin
  Create(AScrs, ACSSs);
  FAttrs := AAttrs;
end;

constructor JSLibReferenceAttribute.Create(const AScrs, ACSSs, AAttrs, ADesc: string);
begin
  Create(AScrs, ACSSs, AAttrs);
  FDesc := ADesc;
end;

{$ENDIF}

{ TWebCustomControl }

constructor TWebCustomControl.Create(ID: string);
begin
  Create(nil);
end;

procedure TWebCustomControl.BeginUpdate;
begin
  //
end;

procedure TWebCustomControl.CMTextChanged(var Message: TMessage);
begin
  Invalidate;
end;

constructor TWebCustomControl.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FBorderStyle := bsSingle;
  FBorderColor := clSilver;
  Color := clWhite;
  FWidthStyle := ssAbsolute;
  FWidthPercent := 100;
  FHeightStyle := ssAbsolute;
  FHeightPercent := 100;
  FElementFont := efProperty;
  FElementPosition := epAbsolute;
  FEventStopPropagation := [eeClick, eeDblClick, eeMouseUp, eeMouseMove, eeMouseDown, eeKeyPress, eeKeyUp, eeKeyDown];
  FDesignTime := (csDesigning in ComponentState) and not
                 ((csReading in AOwner.ComponentState) or (csLoading in AOwner.ComponentState));

  if FDesignTime and (GetOwnerCSSLibrary(AOwner) = cssBootstrap) then
    InitBootstrapStyle;
end;

procedure TWebCustomControl.EndUpdate;
begin
  //
end;

function TWebCustomControl.GetContext: TJSCanvasRenderingContext2D;
begin
  Result := nil;
end;

function TWebCustomControl.GetDesignText: string;
{$IFNDEF LCLTMSWEB}
const
  cCaptionProp = 'CAPTION';
  cTextProp = 'TEXT';
  cValueProp = 'VALUE';
var
  lContext: TRttiContext;
  lType: TRttiType;
  lProp: TRttiProperty;
  lBuffer: string;
begin
  lBuffer := Name;

  lContext := TRttiContext.Create();
  try
    lType := lContext.GetType(Self.ClassType);
    if Assigned(lType) then
    begin
      lProp := lType.GetProperty(cCaptionProp);

      if Assigned(lProp) then
        lBuffer := lProp.GetValue(Self).AsString
      else
      begin
        lProp := lType.GetProperty(cTextProp);
        if Assigned(lProp) then
          lBuffer := lProp.GetValue(Self).AsString
        else
        begin
          lProp := lType.GetProperty(cValueProp);
          if Assigned(lProp) then
          begin
            try
              lBuffer := lProp.GetValue(Self).AsString
            except
              lBuffer := Name;
            end;
          end;
        end;
      end;
    end;

  finally
    lContext.Free;
  end;

  Result := lBuffer;
end;
{$ENDIF}
{$IFDEF LCLTMSWEB}
begin
  Result := ClassName;
end;
{$ENDIF}

function TWebCustomControl.GetIsLinked: boolean;
begin
  Result := false;
end;

procedure TWebCustomControl.InitBootstrapStyle;
begin
  ElementFont := efCSS;
end;

procedure TWebCustomControl.Paint;
var
  lRect: TRect;
  lBuffer: string;
begin
  if Color = clNone then
    Canvas.Brush.Color := clWindow
  else
    Canvas.Brush.Color := Color;

  Canvas.Brush.Style := bsSolid;
  Canvas.Pen.Style := psSolid;

  if BorderStyle = bsSingle then
    Canvas.Pen.Color := BorderColor
  else
    Canvas.Pen.Color := Canvas.Brush.Color;

  lRect := ClientRect;
  Canvas.Rectangle(lRect);

  lBuffer := GetDesignText;

  InflateRect(lRect,-1,-1);

  if (Width > 20) and (Height > 10) then
    DrawText(Canvas.Handle, PChar(lBuffer), Length(lBuffer), lRect, DT_SINGLELINE or DT_END_ELLIPSIS or DT_CENTER or DT_VCENTER
    {$IFNDEF LCLTMSWEB}
    or DT_HIDEPREFIX
    {$ENDIF}
    or DT_END_ELLIPSIS);
end;

procedure TWebCustomControl.PreventDefault;
begin
  //
end;

procedure TWebCustomControl.SetBorderColor(const Value: TColor);
begin
  if (FBorderColor <> Value) then
  begin
    FBorderColor := Value;
    Invalidate;
  end;
end;

procedure TWebCustomControl.SetHeightPercent(const Value: TPercentSize);
begin
  if (FHeightPercent <> Value) then
  begin
    FHeightPercent := Value;
    if Assigned(Parent) then
      Parent.Realign;
  end;
end;

procedure TWebCustomControl.SetHeightStyle(const Value: TSizeStyle);
begin
  if (FHeightStyle <> Value) then
  begin
    FHeightStyle := Value;
    if Assigned(Parent) then
      Parent.Realign;
  end;
end;

{$IFNDEF LCLTMSWEB}
procedure TWebCustomControl.SetParent(AParent: TWinControl);
begin
  inherited;

  if FDesignTime and Assigned(Parent) and not (csDestroying in ComponentState) then
  begin
    if IsRelativeParent(AParent) then
    begin
      ElementPosition := epIgnore;
      ElementFont := efCSS;
      WidthStyle := ssAuto;
      HeightStyle := ssAuto;
    end;
    ChildOrder := AParent.ControlCount - 1;
  end;
end;
{$ENDIF}


procedure TWebCustomControl.SetWidthPercent(const Value: TPercentSize);
begin
  if (FWidthPercent <> Value) then
  begin
    FWidthPercent := Value;
    if Assigned(Parent) then
      Parent.Realign;
  end;
end;

procedure TWebCustomControl.SetWidthStyle(const Value: TSizeStyle);
begin
  if (FWidthStyle <> Value) then
  begin
    FWidthStyle := Value;
    if Assigned(Parent) then
      Parent.Realign;
  end;
end;

{ TWebNonVisualCustomControl }

constructor TWebNonVisualCustomControl.Create(AOwner: TComponent);
begin
  inherited;
  Width := 24;
  Height := 24;
end;

procedure TWebNonVisualCustomControl.Paint;
var
  bmp: TBitmap;
begin
  inherited;

  bmp := TBitmap.Create;
  try
    {$IFNDEF LCLTMSWEB}
    bmp.LoadFromResourceName(FindClassHInstance(ClassInfo), ClassName);
    {$ENDIF}
    {$IFDEF LCLTMSWEB}
    bmp.LoadFromResourceName(HInstance, ClassName);
    {$ENDIF}
    Canvas.Draw(0,0,bmp);
  finally
    bmp.Free;
  end;
end;

procedure TWebNonVisualCustomControl.SetBounds(ALeft, ATop, AWidth,
  AHeight: Integer);
begin
  inherited SetBounds(ALeft, ATop, 24, 24);
end;



{ TWebControl }


function TWebControl.get_AlignWithMargins: Boolean;
begin
  {$IFNDEF LCLTMSWEB}
  Result := inherited AlignWithMargins;
  {$ENDIF}
  {$IFDEF LCLTMSWEB}
  Result := False;
  {$ENDIF}
end;

function TWebControl.get_Cursor: TCursor;
begin
  Result := inherited Cursor;
end;

function TWebControl.get_CustomHint: TCustomHint;
begin
  {$IFNDEF LCLTMSWEB}
  Result := inherited CustomHint;
  {$ENDIF}
  {$IFDEF LCLTMSWEB}
  Result := inherited Hint;
  {$ENDIF}
end;

function TWebControl.get_Height: Integer;
begin
  Result := inherited Height;
end;

function TWebControl.get_HelpContext: THelpContext;
begin
  Result := inherited HelpContext;
end;

function TWebControl.get_HelpKeyword: String;
begin
  Result := inherited HelpKeyword;
end;

function TWebControl.get_HelpType: THelpType;
begin
  Result := inherited HelpType;
end;

function TWebControl.get_Hint: string;
begin
  Result := inherited Hint;
end;

function TWebControl.get_Left: Integer;
begin
  Result := inherited Left;
end;

function TWebControl.get_Margins: TMargins;
begin
  {$IFNDEF LCLTMSWEB}
  Result := inherited Margins;
  {$ENDIF}
  {$IFDEF LCLTMSWEB}
  Result := -1;
  {$ENDIF}
end;

function TWebControl.get_ParentCustomHint: Boolean;
begin
  {$IFNDEF LCLTMSWEB}
  Result := inherited ParentCustomHint;
  {$ENDIF}
  {$IFDEF LCLTMSWEB}
  Result := False;
  {$ENDIF}
end;

function TWebControl.get_Top: Integer;
begin
  Result := inherited Top;
end;

function TWebControl.get_Width: Integer;
begin
  Result := inherited Width;
end;

{ TCustomWebControl }

procedure TCustomWebControl.PreventDefault;
begin

end;


{ TCSSCodeFragment }

constructor TCSSCodeFragment.Create(ACollection: TCollection);
begin
  inherited;
  FCSS := TStringList.Create;
end;

destructor TCSSCodeFragment.Destroy;
begin
  FCSS.Free;
  inherited;
end;

procedure TCSSCodeFragment.SetCSS(const Value: TStringList);
begin
  FCSS.Assign(Value);
end;

{ TCSSCodeFragments }

function TCSSCodeFragments.Add: TCSSCodeFragment;
begin
  Result := TCSSCodeFragment(inherited Add);
end;

constructor TCSSCodeFragments.Create(AOwner: TComponent);
begin
  inherited Create(AOwner, TCSSCodeFragment);
end;

function TCSSCodeFragments.GetItemEx(Index: integer): TCSSCodeFragment;
begin
  Result := TCSSCodeFragment(inherited Items[Index]);
end;

function TCSSCodeFragments.Insert(Index: integer): TCSSCodeFragment;
begin
  Result := TCSSCodeFragment(inherited Insert(Index));
end;

procedure TCSSCodeFragments.SetItemEx(Index: integer;
  const Value: TCSSCodeFragment);
begin
  inherited Items[Index] := Value;
end;

{ TCSSCodeManager }

constructor TCSSCodeManager.Create(AOwner: TComponent);
begin
  inherited;
  FCSSFragments := TCSSCodeFragments.Create(Self);
end;

destructor TCSSCodeManager.Destroy;
begin
  FCSSFragments.Free;
  inherited;
end;

function TCSSCodeManager.GetClassCSS(AClassname: string): string;
var
  CSSCodeFragment: TCSSCodeFragment;
begin
  Result := '';

  CSSCodeFragment := GetClassFragment(AClassName);

  if Assigned(CSSCodeFragment) then
    Result := CSSCodeFragment.CSS.Text;
end;

function TCSSCodeManager.GetClassFragment(AClassname: string): TCSSCodeFragment;
var
  i: integer;
begin
  Result := nil;
  for i := 0 to CSSFragments.Count - 1 do
  begin
    if CSSFragments.Items[i].ControlClassName = AClassname then
      Result := CSSFragments.Items[i];
  end;
end;

function TCSSCodeManager.LoadResourceAsString(AClassName,
  AResourceType: string): string;
var
  ms: TMemoryStream;
  sl: TStringList;
begin
  Result := '';
  ms := TMemoryStream.Create;
  try
    LoadResourceFile(AClassname, AResourceType, ms);

    sl := TStringList.Create;
    sl.LoadFromStream(ms);
    Result := sl.Text;
    sl.Free;

  finally
    ms.Free;
  end;

end;

procedure TCSSCodeManager.LoadResourceFile(AClassName, AResourceType: string;
  ms: TMemoryStream);
var
  HResInfo: HRSRC;
  HGlobal: THandle;
  Buffer, AType : pchar;
begin
  AType := pchar(AResourceType);
  HResInfo := FindResource(HInstance, pchar(AClassName), AType);

  HGlobal := LoadResource(HInstance, HResInfo);
  if HGlobal = 0 then
    raise EResNotFound.Create('Can''t load resource: '+AClassName);

  Buffer := LockResource(HGlobal);
  ms.clear;
  ms.WriteBuffer(Buffer[0], SizeOfResource(HInstance, HResInfo));
  ms.Seek(0,0);
  UnlockResource(HGlobal);
  FreeResource(HGlobal);
end;

procedure TCSSCodeManager.SetCSSFragments(const Value: TCSSCodeFragments);
begin
  FCSSFragments.Assign(Value);
end;

{ TWebCSSControl }

procedure TWebCSSControl.AddControlCSS;
var
  CSSMgr: TCSSCodeManager;
  CSSFragment: TCSSCodeFragment;
  FOwner: TComponent;
  i: integer;
  HasFragment: boolean;
begin
  CSSMgr := nil;

  FOwner := Owner;

  while Assigned(FOwner) and not (FOwner is TCustomForm) do
  begin
    FOwner := FOwner.Owner;
  end;

  if Assigned(FOwner) and (FOwner is TCustomForm) then
  begin
    for i := 0 to (FOwner as TCustomForm).ComponentCount - 1 do
    begin
      if (FOwner as TCustomForm).Components[i] is TCSSCodeManager then
      begin
        CSSMgr := (FOwner as TCustomForm).Components[i] as TCSSCodeManager;
        break;
      end;
    end;
  end;

  HasFragment := false;

  if not Assigned(CSSMgr) then
  begin
    CSSMgr := TCSSCodeManager.Create(Owner);
    CSSMgr.Name := 'CSSCodeManager1';
    CSSFragment := CSSMgr.CSSFragments.Add;
  end
  else
  begin
    CSSFragment := CSSMgr.GetClassFragment(Self.ClassName);
    HasFragment := Assigned(CSSFragment);
    if not HasFragment then
      CSSFragment := CSSMgr.CSSFragments.Add;
  end;

  if not HasFragment then
  begin
    CSSFragment.ControlClassname := Self.ClassName;
    CSSFragment.CSS.Text := CSSMgr.LoadResourceAsString(Self.ClassName, 'CSS');
  end;
end;

{ TWebGraphicControl }

procedure TWebGraphicControl.BeginUpdate;
begin
  //
end;

constructor TWebGraphicControl.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FWidthStyle := ssAbsolute;
  FWidthPercent := 100;
  FHeightStyle := ssAbsolute;
  FHeightPercent := 100;
  FElementFont := efProperty;
  FElementPosition := epAbsolute;
  FEventStopPropagation := [eeClick, eeDblClick, eeMouseUp, eeMouseMove, eeMouseDown, eeKeyPress, eeKeyDown, eeKeyUp];
end;

procedure TWebGraphicControl.EndUpdate;
begin
  //
end;

function TWebGraphicControl.GetIsLinked: boolean;
begin
  Result := false;
end;

procedure TWebGraphicControl.SetHeightPercent(const Value: TPercentSize);
begin
  FHeightPercent := Value;
end;

procedure TWebGraphicControl.SetHeightStyle(const Value: TSizeStyle);
begin
  FHeightStyle := Value;
end;

procedure TWebGraphicControl.SetWidthPercent(const Value: TPercentSize);
begin
  FWidthPercent := Value;
end;

procedure TWebGraphicControl.SetWidthStyle(const Value: TSizeStyle);
begin
  FWidthStyle := Value;
end;

end.
