{********************************************************************}
{                                                                    }
{ written by TMS Software                                            }
{            Copyright (c) 2018 - 2020                               }
{            Email : info@tmssoftware.com                            }
{            Web : http://www.tmssoftware.com                        }
{                                                                    }
{ The source code is given as is. The author is not responsible      }
{ for any possible damage done due to the use of this code.          }
{ The complete source code remains property of the author and may    }
{ not be distributed, published, given or sold in any form as such.  }
{ No parts of the source code can be included in any other component }
{ or application without written authorization of the author.        }
{********************************************************************}

unit WEBLib.DBCtrls;

interface

uses
  {$IFNDEF LCLTMSWEB}
  System.Classes, System.Types, Winapi.Windows, DB, DBClient, Vcl.Grids, Vcl.Graphics
  {$ENDIF}
  {$IFDEF LCLTMSWEB}
  Classes, Types, LCLType, LCLIntf, DB, Grids, Graphics
  {$ENDIF}
  , Web, JS, WEBLib.Controls, WEBLib.StdCtrls, WEBLib.ExtCtrls,
    WEBLib.Mask, WEBlib.Grids, WEBLib.EditAutocomplete
  ;

type
  TWebLinkType = (wlHttp, wlMail, wlHttps, wlFTP);

  TNavigateBtn = (nbFirst, nbPrior, nbNext, nbLast,
                  nbInsert, nbDelete, nbEdit, nbPost, nbCancel);

  TNavButtonSet = set of TNavigateBtn;

  TDBDataLink = class(TDataLink)
  private
    FOnActiveChange: TNotifyEvent;
    FOnDataChange: TNotifyEvent;
  protected
    procedure ActiveChanged; override;
    procedure RecordChanged(Field: TField); override;
  published
    property OnDataChange: TNotifyEvent read FOnDataChange write FOnDataChange;
    property OnActiveChange: TNotifyEvent read FOnActiveChange write FOnActiveChange;
  end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebDBEdit = class(TWebEdit)
  private
    FDataField: string;
    FDataSource: TDataSource;
  protected
    procedure Notification(AComponent: TComponent; AOperation: TOperation); override;
  published
    property DataField: string read FDataField write FDataField;
    property DataSource: TDataSource read FDataSource write FDataSource;
  end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebDBComboBox = class(TWebComboBox)
  private
    FDataField: string;
    FDataSource: TDataSource;
  protected
    procedure Notification(AComponent: TComponent; AOperation: TOperation); override;
  published
    property DataField: string read FDataField write FDataField;
    property DataSource: TDataSource read FDataSource write FDataSource;
  end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebDBLookupComboBox = class(TCustomWebLookupComboBox)
  private
    FDataField: string;
    FDataSource: TDataSource;
    FListSource: TDataSource;
    FListField: string;
    FKeyField: string;
  protected
    procedure Notification(AComponent: TComponent; AOperation: TOperation); override;
  published
    property DataField: string read FDataField write FDataField;
    property DataSource: TDataSource read FDataSource write FDataSource;
    property KeyField: string read FKeyField write FKeyField;
    property ListField: string read FListField write FListField;
    property ListSource: TDataSource read FListSource write FListSource;
  end;


  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebDBEditAutoComplete = class(TWebEditAutoComplete)
  private
    FDataField: string;
    FDataSource: TDataSource;
  protected
    procedure Notification(AComponent: TComponent; AOperation: TOperation); override;
  published
    property DataField: string read FDataField write FDataField;
    property DataSource: TDataSource read FDataSource write FDataSource;
  end;


  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebDBMaskEdit = class(TWebMaskEdit)
  private
    FDataField: string;
    FDataSource: TDataSource;
  protected
    procedure Notification(AComponent: TComponent; AOperation: TOperation); override;
  published
    property DataField: string read FDataField write FDataField;
    property DataSource: TDataSource read FDataSource write FDataSource;
  end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebDBLabel = class(TWebLabel)
  private
    FDataField: string;
    FDataSource: TDataSource;
  protected
    procedure Notification(AComponent: TComponent; AOperation: TOperation); override;
  published
    property DataField: string read FDataField write FDataField;
    property DataSource: TDataSource read FDataSource write FDataSource;
  end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebDBCheckBox = class(TWebCheckBox)
  private
    FDataField: string;
    FDataSource: TDataSource;
    FValueChecked: string;
    FValueUnChecked: string;
  protected
    procedure Notification(AComponent: TComponent; AOperation: TOperation); override;
  published
    property DataField: string read FDataField write FDataField;
    property DataSource: TDataSource read FDataSource write FDataSource;
    property ValueChecked: string read FValueChecked write FValueChecked;
    property ValueUnChecked: string read FValueUnChecked write FValueUnChecked;
  end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebDBSpinEdit = class(TWebSpinEdit)
  private
    FDataField: string;
    FDataSource: TDataSource;
  protected
    procedure Notification(AComponent: TComponent; AOperation: TOperation); override;
  published
    property DataField: string read FDataField write FDataField;
    property DataSource: TDataSource read FDataSource write FDataSource;
  end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebDBDateTimePicker = class(TWebDateTimePicker)
  private
    FDataField: string;
    FDataSource: TDataSource;
  protected
    procedure Notification(AComponent: TComponent; AOperation: TOperation); override;
  published
    property DataField: string read FDataField write FDataField;
    property DataSource: TDataSource read FDataSource write FDataSource;
  end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebDBRadioGroup = class(TWebRadioGroup)
  private
    FDataField: string;
    FDataSource: TDataSource;
    FValues: TStrings;
    procedure SetValues(const Value: TStrings);
  protected
    procedure Notification(AComponent: TComponent; AOperation: TOperation); override;
  public
    constructor Create(AOwner: TComponent); overload; override;
    destructor Destroy; override;
  published
    property DataField: string read FDataField write FDataField;
    property DataSource: TDataSource read FDataSource write FDataSource;
    property Values: TStrings read FValues write SetValues;
  end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebDBLinkLabel = class(TCustomWebLinkLabel)
  private
    FDataField: string;
    FDataSource: TDataSource;
    FLinkType: TWebLinkType;
  protected
    procedure Notification(AComponent: TComponent; AOperation: TOperation); override;
  published
    property Caption;
    property DataField: string read FDataField write FDataField;
    property DataSource: TDataSource read FDataSource write FDataSource;
    property LinkType: TWebLinkType read FLinkType write FLinkType;
  end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebDBMemo = class(TWebMemo)
  private
    FDataField: string;
    FDataSource: TDataSource;
  protected
    procedure Notification(AComponent: TComponent; AOperation: TOperation); override;
  published
    property DataField: string read FDataField write FDataField;
    property DataSource: TDataSource read FDataSource write FDataSource;
  end;

  TSetURLEvent = procedure(Sender: TObject; var AURL: string) of object;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebDBImageControl = class(TWebCustomImageControl)
  private
    FDataField: string;
    FDataSource: TDataSource;
    FBaseURL: string;
    FOnSetURL: TSetURLEvent;
  protected
    procedure Notification(AComponent: TComponent; AOperation: TOperation); override;
  published
    property BaseURL: string read FBaseURL write FBaseURL;
    property DataField: string read FDataField write FDataField;
    property DataSource: TDataSource read FDataSource write FDataSource;
    property OnSetURL: TSetURLEvent read FOnSetURL write FOnSetURL;
  end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebDBNavigator = class(TWebCustomControl)
  private
    FDataSource: TDataSource;
    FVisibleButtons: TNavButtonSet;
    FHints: TStringList;
    procedure SetHints(const Value: TStringList);
  protected
    procedure Notification(AComponent: TComponent; AOperation: TOperation); override;
  public
    constructor Create(AOwner: TComponent); overload; override;
    destructor Destroy; override;
  published
    property Align;
    property Anchors;

    property DataSource: TDataSource read FDataSource write FDataSource;
    property ElementPosition;
    property Hints: TStringList read FHints write SetHints;
    property Visible;
    property VisibleButtons: TNavButtonSet read FVisibleButtons write FVisibleButtons default [nbFirst, nbPrior, nbNext, nbLast, nbInsert, nbDelete, nbEdit, nbPost, nbCancel];
  end;

  TWebDBTableControl = class;

  TColumnDataType = (cdText, cdImage, cdLink, cdCheck, cdRadio, cdExternalLink);

  TWebTableControlColumn = class(TCollectionItem)
  private
    FDataField: string;
    FTitle: string;
    FDataType: TColumnDataType;
    FClassName: string;
    FTitleClassName: string;
    procedure SetDataField(const Value: string);
    procedure SetTitle(const Value: string);
  protected
    function GetDisplayName: string; override;
    function TableControl: TWebDBTableControl;
  public
    procedure Assign(Source: TPersistent); override;
  published
    property ElementClassName: string read FClassName write FClassName;
    property DataField: string read FDataField write SetDataField;
    property DataType: TColumnDataType read FDataType write FDataType default cdText;
    property Title: string read FTitle write SetTitle;
    property TitleElementClassName: string read FTitleClassName write FTitleClassName;
  end;

  TWebTableControlColumns = class(TOwnedCollection)
  private
    FOnChange: TNotifyEvent;
    function GetItem(Index: integer): TWebTableControlColumn;
    procedure SetItem(Index: integer; const Value: TWebTableControlColumn);
  protected
    procedure Update(Item: TCollectionItem); override;
    procedure DoChange; virtual;
  public
    constructor Create(AOwner: TComponent);
    function Add: TWebTableControlColumn;
    function Insert(Index: integer): TWebTableControlColumn;
    property Items[Index: integer]: TWebTableControlColumn read GetItem write SetItem; default;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
  end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebDBTableControl = class(TWebCustomTableControl)
  private
    FDataLink: TDBDataLink;
    FColumns: TWebTableControlColumns;
    procedure SetColumns(const Value: TWebTableControlColumns);
  protected
    procedure ColumnsChanged(Sender: TObject);
    procedure DataChange(Sender: TObject);
    procedure ActiveChange(Sender: TObject);
    procedure SetDataSource(const Value: TDataSource);
    function GetDataSource: TDataSource;
    procedure Notification(AComponent: TComponent; AOperation: TOperation); override;
    function HasDefinedFields: boolean;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property Columns: TWebTableControlColumns read FColumns write SetColumns;
    property DataSource: TDataSource read GetDataSource write SetDataSource;
    property ElementFont;
    property ElementClassName;
    property ElementId;
    property ElementPosition;
  end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebDBResponsiveGrid = class(TWebResponsiveGrid)
  private
    FDataLink: TDBDataLink;
  protected
    procedure DataChange(Sender: TObject);
    procedure ActiveChange(Sender: TObject);
    procedure SetDataSource(const Value: TDataSource);
    function GetDataSource: TDataSource;
    procedure Notification(AComponent: TComponent; AOperation: TOperation); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property DataSource: TDataSource read GetDataSource write SetDataSource;
    property ElementFont;
    property ElementClassName;
    property ElementId;
    property ElementPosition;
  end;

  TWebDBGrid = class;

  TGridSortIndicator = (siNone, siAscending, siDescending);


  TWebGridColumn = class(TCollectionItem)
  private
    FDataField: string;
    FTitle: string;
    FWidth: integer;
    FDataType: TColumnDataType;
    FClassName: TElementClassName;
    FSortIndicator: TGridSortIndicator;
    FTitleClassName: TElementClassName;
    procedure SetDataField(const Value: string);
    procedure SetTitle(const Value: string);
    procedure SetWidth(const Value: integer);
  protected
    function GetDisplayName: string; override;
    function Grid: TWebDBGrid;
  public
    procedure Assign(Source: TPersistent); override;
    constructor Create(ACollection: TCollection); override;
    property SortIndicator: TGridSortIndicator read FSortIndicator write FSortIndicator default siNone;
  published
    property ElementClassName: TElementClassName read FClassName write FClassName;
    property DataField: string read FDataField write SetDataField;
    property DataType: TColumnDataType read FDataType write FDataType default cdText;
    property Title: string read FTitle write SetTitle;
    property TitleElementClassName: TElementClassName read FTitleClassName write FTitleClassName;
    property Width: integer read FWidth write SetWidth default 64;
  end;

  TWebGridColumns = class(TOwnedCollection)
  private
    FOnChange: TNotifyEvent;
    function GetItem(Index: integer): TWebGridColumn;
    procedure SetItem(Index: integer; const Value: TWebGridColumn);
  protected
    procedure Update(Item: TCollectionItem); override;
    procedure DoChange; virtual;
  public
    constructor Create(AOwner: TComponent);
    function Add: TWebGridColumn;
    function Insert(Index: integer): TWebGridColumn;
    property Items[Index: integer]: TWebGridColumn read GetItem write SetItem; default;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
  end;

  TGetEditEvent = procedure (Sender: TObject; ACol, ARow: Longint; var Value: string) of object;
  TSetEditEvent = procedure (Sender: TObject; ACol, ARow: Longint; const Value: string) of object;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebDBGrid = class(TCustomGrid)
  private
    FDataLink: TDBDataLink;
    FColumns: TWebGridColumns;
    FColWidthUpdating: boolean;
    FOnGetCellClass: TGridGetCellClassEvent;
    FOnGetCellData: TGridGetCellDataEvent;
    FOnGetCellChildren: TGridGetCellChildrenEvent;
    FElementFont: TElementFont;
    FElementId: TElementID;
    FElementPosition: TElementPosition;
    FElementClassname: TElementClassName;
    FDefaultRowHeight: integer;
    FOnCheckClick: TCellCheckEvent;
    FWidthStyle: TSizeStyle;
    FHeightPercent: TPercentSize;
    FHeightStyle: TSizeStyle;
    FWidthPercent: TPercentSize;
    FShowIndicator: boolean;
    FOnValidateEdit: TGetEditEvent;
    FOnGetEditText: TGetEditEvent;
    FOnSetEditText: TSetEditEvent;
    FOnCanEditCell: TCellCanEditEvent;
    FFixedTextColor: TColor;
    FOnColumnSized: TColumnSizedEvent;
    {$IFDEF LCLTMSWEB}
    FOnFixedCellClick: TCellEvent;
    {$ENDIF}
    procedure SetColumns(const Value: TWebGridColumns);
    procedure SetHeightPercent(const Value: TPercentSize);
    procedure SetHeightStyle(const Value: TSizeStyle);
    procedure SetWidthPercent(const Value: TPercentSize);
    procedure SetWidthStyle(const Value: TSizeStyle);
    function GetFixedColsEx: integer;
    procedure SetFixedColsEx(const Value: integer);
    procedure SetDefaultRowHeight(const Value: integer);
    function GetRowHeights(AIndex: integer): integer;
    procedure SetRowHeights(AIndex: integer; const Value: integer);
  protected
    procedure ColWidthsChanged; override;
    procedure DrawCell(ACol, ARow: Longint; ARect: TRect;
      AState: TGridDrawState); override;
    procedure DataChange(Sender: TObject);
    procedure ActiveChange(Sender: TObject);
    procedure SetDataSource(const Value: TDataSource);
    function GetDataSource: TDataSource;
    procedure ColumnsChanged(Sender: TObject); virtual;
    procedure Notification(AComponent: TComponent; AOperation: TOperation); override;
    function HasDefinedFields: boolean;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure BeginUpdate;
    procedure EndUpdate;
    property RowHeights[AIndex: integer]: integer read GetRowHeights write SetRowHeights;
    procedure LoadFromJSON(const AURL: string; ADataNode: string = ''); overload;
    procedure LoadFromJSON(AJSON: TJSObject; ADataNode: string = ''); overload;
    procedure LoadFromCSV(const AURL: string; Delimiter: char = ';'; LoadFixed: boolean = false);
    procedure SaveToCSV(const AFileName: string; Delimiter: char = ';'; SaveFixed: boolean = false);

    property ColWidths;
    property Selection;
  published
    property Align;
    property Anchors;
    property BorderStyle;
    property Color;
    property Columns: TWebGridColumns read FColumns write SetColumns;
    property DefaultRowHeight: integer read FDefaultRowHeight write SetDefaultRowHeight default 22;
    property DataSource: TDataSource read GetDataSource write SetDataSource;
    property DragMode;

    property ElementFont: TElementFont read FElementFont write FElementFont default efProperty;
    property ElementClassName: TElementClassName read FElementClassname write FElementClassName;
    property ElementId: TElementID read FElementId write FElementId;
    property ElementPosition: TElementPosition read FElementPosition write FElementPosition default epAbsolute;

    property FixedColor;
    property FixedTextColor: TColor read FFixedTextColor write FFixedTextColor default clBlack;
    property FixedCols: integer read GetFixedColsEx write SetFixedColsEx;
    property FixedRows;
    property Font;

    property Options;
    property ParentFont;
    property TabOrder;
    {$IFNDEF LCLTMSWEB}
    property Touch;
    {$ENDIF}

    property HeightStyle: TSizeStyle read FHeightStyle write SetHeightStyle default ssAbsolute;
    property HeightPercent: TPercentSize read FHeightPercent write SetHeightPercent default 100;

    property ShowIndicator: boolean read FShowIndicator write FShowIndicator default true;
    {$IFNDEF LCLTMSWEB}
    property StyleElements;
    {$ENDIF}
    property Visible;
    property WidthStyle: TSizeStyle read FWidthStyle write SetWidthStyle default ssAbsolute;
    property WidthPercent: TPercentSize read FWidthPercent write SetWidthPercent default 100;


    property OnCanEditCell: TCellCanEditEvent read FOnCanEditCell write FOnCanEditCell;
    property OnCheckClick: TCellCheckEvent read FOnCheckClick write FOnCheckClick;
    property OnColumnSized: TColumnSizedEvent read FOnColumnSized write FOnColumnSized;
    property OnClick;
    property OnDblClick;
    property OnEnter;
    property OnExit;
    {$IFNDEF LCLTMSWEB}
    property OnFixedCellClick;
    {$ENDIF}
    {$IFDEF LCLTMSWEB}
    property OnFixedCellClick: TCellEvent read FOnFixedCellClick write FOnFixedCellClick;
    {$ENDIF}
    property OnGetCellChildren: TGridGetCellChildrenEvent read FOnGetCellChildren write FOnGetCellChildren;
    property OnGetCellData: TGridGetCellDataEvent read FOnGetCellData write FOnGetCellData;
    property OnGetCellClass: TGridGetCellClassEvent read FOnGetCellClass write FOnGetCellClass;
    property OnGetEditText: TGetEditEvent read FOnGetEditText write FOnGetEditText;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    {$IFNDEF LCLTMSWEB}
    property OnMouseActivate;
    {$ENDIF}
    property OnMouseDown;
    property OnMouseEnter;
    property OnMouseLeave;
    property OnMouseMove;
    property OnMouseUp;
    property OnMouseWheelDown;
    property OnMouseWheelUp;
    property OnSetEditText: TSetEditEvent read FOnSetEditText write FOnSetEditText;
    property OnValidateEdit: TGetEditEvent read FOnValidateEdit write FOnValidateEdit;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnStartDrag;
  end;

implementation

uses
  SysUtils;

{ TWebDBRadioGroup }

constructor TWebDBRadioGroup.Create(AOwner: TComponent);
begin
  inherited;
  FValues := TStringList.Create;
end;

destructor TWebDBRadioGroup.Destroy;
begin
  FValues.Free;
  inherited;
end;

procedure TWebDBRadioGroup.Notification(AComponent: TComponent;
  AOperation: TOperation);
begin
  inherited;
  if (AOperation = opRemove) and (AComponent = FDataSource) then
    FDataSource := nil;
end;

procedure TWebDBRadioGroup.SetValues(const Value: TStrings);
begin
  FValues.Assign(Value);
end;

{ TWebDBNavigator }

constructor TWebDBNavigator.Create(AOwner: TComponent);
begin
  inherited;
  Width := 330;
  Height := 25;
  FVisibleButtons := [nbFirst, nbPrior, nbNext, nbLast,
                  nbInsert, nbDelete, nbEdit, nbPost, nbCancel];

  FHints := TStringList.Create;
  FHints.Add('First');
  FHints.Add('Prior');
  FHints.Add('Next');
  FHints.Add('Last');
  FHints.Add('Edit');
  FHints.Add('Post');
  FHints.Add('Insert');
  FHints.Add('Delete');
  FHints.Add('Cancel');

end;

destructor TWebDBNavigator.Destroy;
begin
  FHints.Free;
  inherited;
end;

procedure TWebDBNavigator.Notification(AComponent: TComponent;
  AOperation: TOperation);
begin
  inherited;
  if (AOperation = opRemove) and (AComponent = FDataSource) then
    FDataSource := nil;
end;

procedure TWebDBNavigator.SetHints(const Value: TStringList);
begin
  FHints.Assign(Value);
end;

{ TWebTableControlColumns }

function TWebTableControlColumns.Add: TWebTableControlColumn;
begin
  Result := TWebTableControlColumn(inherited Add);
end;

procedure TWebTableControlColumns.DoChange;
begin
  inherited;
  if Assigned(OnChange) then
    OnChange(Self);
end;

constructor TWebTableControlColumns.Create(AOwner: TComponent);
begin
  inherited Create(AOwner, TWebTableControlColumn);
end;

function TWebTableControlColumns.GetItem(Index: integer): TWebTableControlColumn;
begin
  Result := TWebTableControlColumn(inherited Items[Index]);
end;

function TWebTableControlColumns.Insert(Index: integer): TWebTableControlColumn;
begin
  Result := TWebTableControlColumn(inherited Insert(Index));
end;

procedure TWebTableControlColumns.SetItem(Index: integer;
  const Value: TWebTableControlColumn);
begin
  inherited Items[Index] := Value;
end;

procedure TWebTableControlColumns.Update(Item: TCollectionItem);
begin
  inherited;
  DoChange;
end;

{ TWebDBTableControl }

procedure TWebDBTableControl.ActiveChange(Sender: TObject);
var
  i: Integer;
begin
  if Columns.Count = 0 then
  begin
    if Assigned(FDataLink.DataSet) then
    begin
      for i := 0 to FDataLink.DataSet.FieldCount - 1 do
      begin
        with Columns.Add do
        begin
          DataField := FDataLink.DataSet.Fields[i].FieldName;
          Title := FDataLink.DataSet.Fields[i].FieldName;
        end;
      end;
    end;
  end;
end;


procedure TWebDBTableControl.ColumnsChanged(Sender: TObject);
begin
  ColCount := Columns.Count;
end;

constructor TWebDBTableControl.Create(AOwner: TComponent);
begin
  inherited;
  FDataLink := TDBDataLink.Create;
  FColumns := TWebTableControlColumns.Create(Self);
  FDataLink.OnActiveChange := ActiveChange;
  FColumns.OnChange := ColumnsChanged;
  ColCount := 0;
end;

procedure TWebDBTableControl.DataChange(Sender: TObject);
begin
  //
end;

destructor TWebDBTableControl.Destroy;
begin
  FColumns.Free;
  FDataLink.Free;
  inherited;
end;

function TWebDBTableControl.GetDataSource: TDataSource;
begin
  Result := FDataLink.DataSource;
end;

function TWebDBTableControl.HasDefinedFields: boolean;
var
  i: integer;
begin
  Result := false;

  for i := 0 to Columns.Count - 1 do
  begin
    if (Columns[i].DataField <> '') then
    begin
      Result := true;
      break;
    end;
  end;
end;

procedure TWebDBTableControl.Notification(AComponent: TComponent;
  AOperation: TOperation);
var
  i: integer;
begin
  inherited;
  if (AOperation = opRemove) and (AComponent = FDataLink.DataSource) then
  begin
    FDataLink.DataSource := nil;
    for i := 0 to Columns.Count - 1 do
      Columns[i].DataField := '';
  end;
end;

procedure TWebDBTableControl.SetColumns(const Value: TWebTableControlColumns);
begin
  FColumns.Assign(Value);
end;

procedure TWebDBTableControl.SetDataSource(const Value: TDataSource);
var
  i: integer;
  col: TWebTableControlColumn;

begin
  FDataLink.DataSource := Value;

  if Assigned(Value) then
  begin
    if Assigned(Value.DataSet) and not HasDefinedFields then
    begin
      for i := 0 to Value.DataSet.FieldCount - 1 do
      begin
        if Columns.Count > i then
          col := Columns[i]
        else
          col := Columns.Add;

        col.DataField := Value.DataSet.Fields[i].FieldName;
        col.Title := Value.DataSet.Fields[i].DisplayLabel;
      end;
    end;
  end;
end;

{ TWebTableControlColumn }

function TWebTableControlColumn.GetDisplayName: string;
begin
  if DataField <> '' then
    Result := DataField
  else
    Result := 'Column ' + IntToStr(Index);
end;

procedure TWebTableControlColumn.SetDataField(const Value: string);
var
  fld: TField;
begin
  if (FDataField <> Value) then
  begin
    FDataField := Value;
    (Collection as TWebTableControlColumns).DoChange;

    if Assigned(TableControl.DataSource) and Assigned(TableControl.DataSource.DataSet) and (Title = '') then
    begin
      fld := TableControl.DataSource.DataSet.FieldByName(Value);
      if Assigned(fld) then
        Title := fld.DisplayLabel;
    end;
  end;
end;

procedure TWebTableControlColumn.SetTitle(const Value: string);
begin
  if (FTitle <> Value) then
  begin
    FTitle := Value;
    (Collection as TWebTableControlColumns).DoChange;
  end;
end;


function TWebTableControlColumn.TableControl: TWebDBTableControl;
begin
  Result := (Collection as TWebTableControlColumns).Owner as TWebDBTableControl;
end;

procedure TWebTableControlColumn.Assign(Source: TPersistent);
begin
  if (Source is TWebTableControlColumn) then
  begin
    FClassName := (Source as TWebTableControlColumn).ElementClassName;
    FDataField := (Source as TWebTableControlColumn).DataField;
    FTitle := (Source as TWebTableControlColumn).Title;
    FTitleClassName := (Source as TWebTableControlColumn).TitleElementClassName;
    FDataType := (Source as TWebTableControlColumn).DataType;
  end;
end;

{ TDBResponsiveGrid }

procedure TWebDBResponsiveGrid.ActiveChange(Sender: TObject);
begin
  //
end;

constructor TWebDBResponsiveGrid.Create(AOwner: TComponent);
begin
  inherited;
  FDataLink := TDBDataLink.Create;
end;

procedure TWebDBResponsiveGrid.DataChange(Sender: TObject);
begin
  //
end;

destructor TWebDBResponsiveGrid.Destroy;
begin
  FDataLink.Free;
  inherited;
end;

function TWebDBResponsiveGrid.GetDataSource: TDataSource;
begin
  Result := FDataLink.DataSource;
end;

procedure TWebDBResponsiveGrid.Notification(AComponent: TComponent;
  AOperation: TOperation);
begin
  inherited;
  if (AOperation = opRemove) and (AComponent = FDataLink.DataSource) then
    FDataLink.DataSource := nil;
end;

procedure TWebDBResponsiveGrid.SetDataSource(const Value: TDataSource);
begin
  FDataLink.DataSource := Value;
end;

{ TWebGridColumn }

procedure TWebGridColumn.Assign(Source: TPersistent);
begin
  if (Source is TWebTableControlColumn) then
  begin
    FClassName := (Source as TWebGridColumn).ElementClassName;
    FDataField := (Source as TWebGridColumn).DataField;
    FTitle := (Source as TWebGridColumn).Title;
    FTitleClassName := (Source as TWebGridColumn).TitleElementClassName;
    FWidth := (Source as TWebGridColumn).Width;
    FDataType := (Source as TWebGridColumn).DataType;
  end;
end;

constructor TWebGridColumn.Create(ACollection: TCollection);
begin
  inherited;
  FWidth := 64;
  FDataType := cdText;
end;

function TWebGridColumn.GetDisplayName: string;
begin
  if DataField <> '' then
    Result := DataField
  else
    Result := 'Column ' + IntToStr(Index);
end;

function TWebGridColumn.Grid: TWebDBGrid;
begin
  Result := (Collection as TWebGridColumns).Owner as TWebDBGrid;
end;

procedure TWebGridColumn.SetDataField(const Value: string);
var
  fld: TField;
begin
  if (FDataField <> Value) then
  begin
    FDataField := Value;
    (Collection as TWebGridColumns).DoChange;

    if Assigned(Grid.DataSource) and Assigned(Grid.DataSource.DataSet) and (Title = '') then
    begin
      fld := Grid.DataSource.DataSet.FieldByName(Value);
      if Assigned(fld) then
        Title := fld.DisplayLabel;
    end;
  end;
end;

procedure TWebGridColumn.SetTitle(const Value: string);
begin
  if (FTitle <> Value) then
  begin
    FTitle := Value;
    (Collection as TWebGridColumns).DoChange;
  end;
end;

procedure TWebGridColumn.SetWidth(const Value: integer);
begin
  if (FWidth <> Value) then
  begin
    FWidth := Value;
    (Collection as TWebGridColumns).DoChange;
  end;
end;

{ TWebGridColumns }

function TWebGridColumns.Add: TWebGridColumn;
begin
  Result := TWebGridColumn(inherited Add);
end;

procedure TWebGridColumns.DoChange;
begin
  inherited;
  if Assigned(OnChange) then
    OnChange(Self);
end;

constructor TWebGridColumns.Create(AOwner: TComponent);
begin
  inherited Create(AOwner, TWebGridColumn);
end;

function TWebGridColumns.GetItem(Index: integer): TWebGridColumn;
begin
  Result := TWebGridColumn(inherited Items[Index]);
end;

function TWebGridColumns.Insert(Index: integer): TWebGridColumn;
begin
  Result := TWebGridColumn(inherited Insert(Index));
end;

procedure TWebGridColumns.SetItem(Index: integer;
  const Value: TWebGridColumn);
begin
  inherited Items[Index] := Value;
end;

procedure TWebGridColumns.Update(Item: TCollectionItem);
begin
  inherited;
  DoChange;
end;


{ TWebDBGrid }

procedure TWebDBGrid.ActiveChange(Sender: TObject);
var
  i: Integer;
begin
  if Columns.Count = 0 then
  begin
    if Assigned(FDataLink.DataSet) then
    begin
      for i := 0 to FDataLink.DataSet.FieldCount - 1 do
      begin
        with Columns.Add do
        begin
          DataField := FDataLink.DataSet.Fields[i].FieldName;
          Title := FDataLink.DataSet.Fields[i].FieldName;
        end;
      end;
    end;
  end;
end;

procedure TWebDBGrid.BeginUpdate;
begin
 //
end;

procedure TWebDBGrid.ColumnsChanged(Sender: TObject);
var
  c: integer;
begin
  ColCount :=  FixedCols + Columns.Count;

  FColWidthUpdating := true;

  // resync col widths
  for c := 0 to Columns.Count - 1 do
    ColWidths[FixedCols + c] := Columns[c].Width;

  FColWidthUpdating := false;

  Invalidate;
end;

procedure TWebDBGrid.ColWidthsChanged;
var
  c: integer;
begin
  inherited;

  if FColWidthUpdating then
    Exit;

  for c := 0 to Columns.Count - 1 do
  begin
    if c + FixedCols < ColCount then
      Columns[c].FWidth := ColWidths[c + FixedCols];
  end;
end;

constructor TWebDBGrid.Create(AOwner: TComponent);
var
  FDesignTime: boolean;
  i: integer;
begin
  inherited;

  ColCount := 5;
  RowCount := 5;
  DefaultRowHeight := 22;

  FDataLink := TDBDataLink.Create;
  FDataLink.OnActiveChange := ActiveChange;
  FColumns := TWebGridColumns.Create(Self);

  FDesignTime := (csDesigning in ComponentState) and not
                 ((csReading in Owner.ComponentState) or (csLoading in Owner.ComponentState));

  FElementFont := efProperty;
  FElementPosition := epAbsolute;
  FShowIndicator := true;

  if FDesignTime then
  begin
    for i := 0 to 3 do
      FColumns.Add;

    ColWidths[0] := 24;
  end;

  FColumns.OnChange := ColumnsChanged;

  FWidthStyle := ssAbsolute;
  FWidthPercent := 100;
  FHeightStyle := ssAbsolute;
  FHeightPercent := 100;
  FFixedTextColor := clBlack;
end;

procedure TWebDBGrid.DataChange(Sender: TObject);
begin
  //
end;

destructor TWebDBGrid.Destroy;
begin
  FDataLink.Free;
  FColumns.Free;

  inherited;
end;

procedure TWebDBGrid.DrawCell(ACol, ARow: Longint; ARect: TRect;
  AState: TGridDrawState);
var
  s: string;
begin
  inherited;

  if (ARow = 0) and (ACol >= FixedCols) and (ACol - FixedCols < Columns.Count) then
  begin
    s := Columns[ACol - FixedCols].Title;
    DrawText(Canvas.Handle, PChar(s), Length(s), ARect, DT_LEFT or DT_END_ELLIPSIS or DT_SINGLELINE or DT_VCENTER);
  end;

  if (ARow = FixedRows) and (ACol >= FixedCols) and (ACol - FixedCols < Columns.Count) then
  begin
    s := '(' + Columns[ACol - FixedCols].DataField + ')';
    DrawText(Canvas.Handle, PChar(s), Length(s), ARect, DT_LEFT or DT_END_ELLIPSIS or DT_SINGLELINE or DT_VCENTER);
  end;
end;

procedure TWebDBGrid.EndUpdate;
begin
 //
end;

function TWebDBGrid.GetDataSource: TDataSource;
begin
  Result := FDataLink.DataSource;
end;

function TWebDBGrid.GetFixedColsEx: integer;
begin
  Result := inherited FixedCols;
end;

function TWebDBGrid.GetRowHeights(AIndex: integer): integer;
begin
  Result := 0;
end;

function TWebDBGrid.HasDefinedFields: boolean;
var
  i: integer;
begin
  Result := false;

  for i := 0 to Columns.Count - 1 do
  begin
    if (Columns[i].DataField <> '') then
    begin
      Result := true;
      break;
    end;
  end;
end;

procedure TWebDBGrid.LoadFromCSV(const AURL: string; Delimiter: char;
  LoadFixed: boolean);
begin
//
end;

procedure TWebDBGrid.LoadFromJSON(const AURL: string; ADataNode: string);
begin
//
end;

procedure TWebDBGrid.LoadFromJSON(AJSON: TJSObject; ADataNode: string);
begin
//
end;

procedure TWebDBGrid.Notification(AComponent: TComponent;
  AOperation: TOperation);
var
  i: integer;
begin
  inherited;
  if (AOperation = opRemove) and (AComponent = FDataLink.DataSource) then
  begin
    FDataLink.DataSource := nil;
    for i := 0 to Columns.Count - 1 do
      Columns[i].DataField := '';
  end;
end;

procedure TWebDBGrid.SaveToCSV(const AFileName: string; Delimiter: char;
  SaveFixed: boolean);
begin
//
end;

procedure TWebDBGrid.SetColumns(const Value: TWebGridColumns);
begin
  FColumns.Assign(Value);
end;

procedure TWebDBGrid.SetDataSource(const Value: TDataSource);
var
  i: integer;
  FCol: TWebGridColumn;
begin
  FDataLink.DataSource := Value;

  if Assigned(Value) then
  begin
    if Assigned(Value.DataSet) and not HasDefinedFields then
    begin
      for i := 0 to Value.DataSet.FieldCount - 1 do
      begin
        if Columns.Count > i then
          FCol := Columns[i]
        else
          FCol := Columns.Add;

        FCol.DataField := Value.DataSet.Fields[i].FieldName;
        FCol.Title := Value.DataSet.Fields[i].DisplayLabel;
      end;
    end;
  end;

end;

procedure TWebDBGrid.SetDefaultRowHeight(const Value: integer);
begin
  FDefaultRowHeight := Value;
  TStringGrid(Self).DefaultRowHeight := Value;
end;

procedure TWebDBGrid.SetFixedColsEx(const Value: integer);
begin
  inherited FixedCols := Value;
  ColumnsChanged(Self);
end;

procedure TWebDBGrid.SetHeightPercent(const Value: TPercentSize);
begin
  FHeightPercent := Value;
end;

procedure TWebDBGrid.SetHeightStyle(const Value: TSizeStyle);
begin
  FHeightStyle := Value;
end;

procedure TWebDBGrid.SetRowHeights(AIndex: integer; const Value: integer);
begin
  //
end;

procedure TWebDBGrid.SetWidthPercent(const Value: TPercentSize);
begin
  FWidthPercent := Value;
end;

procedure TWebDBGrid.SetWidthStyle(const Value: TSizeStyle);
begin
  FWidthStyle := Value;
end;

{ TDBDataLink }

procedure TDBDataLink.ActiveChanged;
begin
  inherited;
  if Assigned(OnActiveChange) then
    OnActiveChange(Self);
end;

procedure TDBDataLink.RecordChanged(Field: TField);
begin
  inherited;
  if Assigned(OnDataChange) then
    OnDataChange(Self);
end;

{ TWebDBEdit }

procedure TWebDBEdit.Notification(AComponent: TComponent;
  AOperation: TOperation);
begin
  inherited;
  if (AOperation = opRemove) and (AComponent = FDataSource) then
    FDataSource := nil;
end;

{ TWebDBLabel }

procedure TWebDBLabel.Notification(AComponent: TComponent;
  AOperation: TOperation);
begin
  inherited;
  if (AOperation = opRemove) and (AComponent = FDataSource) then
    FDataSource := nil;
end;

{ TWebDBCheckBox }

procedure TWebDBCheckBox.Notification(AComponent: TComponent;
  AOperation: TOperation);
begin
  inherited;
  if (AOperation = opRemove) and (AComponent = FDataSource) then
    FDataSource := nil;
end;

{ TWebDBSpinEdit }

procedure TWebDBSpinEdit.Notification(AComponent: TComponent;
  AOperation: TOperation);
begin
  inherited;
  if (AOperation = opRemove) and (AComponent = FDataSource) then
    FDataSource := nil;
end;

{ TWebDBDateTimePicker }

procedure TWebDBDateTimePicker.Notification(AComponent: TComponent;
  AOperation: TOperation);
begin
  inherited;
  if (AOperation = opRemove) and (AComponent = FDataSource) then
    FDataSource := nil;
end;

{ TWebDBLinkLabel }

procedure TWebDBLinkLabel.Notification(AComponent: TComponent;
  AOperation: TOperation);
begin
  inherited;
  if (AOperation = opRemove) and (AComponent = FDataSource) then
    FDataSource := nil;
end;

{ TWebDBMemo }

procedure TWebDBMemo.Notification(AComponent: TComponent;
  AOperation: TOperation);
begin
  inherited;
  if (AOperation = opRemove) and (AComponent = FDataSource) then
    FDataSource := nil;
end;

{ TWebDBImageControl }

procedure TWebDBImageControl.Notification(AComponent: TComponent;
  AOperation: TOperation);
begin
  inherited;
  if (AOperation = opRemove) and (AComponent = FDataSource) then
    FDataSource := nil;
end;

{ TWebDBMaskEdit }

procedure TWebDBMaskEdit.Notification(AComponent: TComponent;
  AOperation: TOperation);
begin
  inherited;
  if (AOperation = opRemove) and (AComponent = FDataSource) then
    FDataSource := nil;
end;

{ TWebDBComboBox }

procedure TWebDBComboBox.Notification(AComponent: TComponent;
  AOperation: TOperation);
begin
  inherited;
  if (AOperation = opRemove) and (AComponent = FDataSource) then
    FDataSource := nil;
end;

{ TWebDBEditAutoComplete }

procedure TWebDBEditAutoComplete.Notification(AComponent: TComponent;
  AOperation: TOperation);
begin
  inherited;
  if (AOperation = opRemove) and (AComponent = FDataSource) then
    FDataSource := nil;
end;

{ TWebDBLookupComboBox }

procedure TWebDBLookupComboBox.Notification(AComponent: TComponent;
  AOperation: TOperation);
begin
  inherited;

  if (AOperation = opRemove) and (AComponent = FDataSource) then
    FDataSource := nil;

  if (AOperation = opRemove) and (AComponent = FListSource) then
    FListSource := nil;
end;

end.
