{********************************************************************}
{                                                                    }
{ written by TMS Software                                            }
{            Copyright (c) 2018                                      }
{            Email : info@tmssoftware.com                            }
{            Web : http://www.tmssoftware.com                        }
{                                                                    }
{ The source code is given as is. The author is not responsible      }
{ for any possible damage done due to the use of this code.          }
{ The complete source code remains property of the author and may    }
{ not be distributed, published, given or sold in any form as such.  }
{ No parts of the source code can be included in any other component }
{ or application without written authorization of the author.        }
{********************************************************************}

unit WEBLib.JQGrid;

interface

uses
  {$IFNDEF LCLTMSWEB}
  System.Classes, Winapi.Windows, RTTI, VCL.Controls, VCL.Forms, VCL.StdCtrls, SysUtils, VCL.Graphics, Types,
  Vcl.Themes, Vcl.Menus, WEBLib.Controls, WEBLib.StdCtrls, WEBLib.Menus, WEBLib.JQCtrls, js;
  {$ENDIF}
  {$IFDEF LCLTMSWEB}
  Classes, Controls, Forms, StdCtrls, SysUtils, Graphics, Types,
  Menus, WEBLib.Controls, WEBLib.StdCtrls, WEBLib.Menus, WEBLib.JQCtrls, js;
  {$ENDIF}

type
  TJQXGridColumnType = (gctDefault, gctEmail, gctLink, gctImage);
  TJQXGridEditor = (geEdit, geCheckBox, geDateTimeInput, geDropDownList, geNumberInput, geNone);
  TJQXGridSelection = (gsSingleCell, gsSingleRow, gsMultipleCells, gsMultipleRows);
//  TJQXGridScrollMode = (gsmPixelScrolling, gsmItemScrolling);
  TJQXGridSortDirection = (gsdAscending, gsdDescending, gsdUnsorted);
  TJQXGridDataType = (gdtArray, gdtCSV, gdtJSON, gdtNone);
  TJQXGridColumnDataType = (gcdString, gcdInteger, gcdDouble, gcdDate);

  TJQXCellEventArgs = class(TPersistent)
  private
    FARow: Integer;
    FACol: Integer;
    FADataField: string;
    FAValue: string;
    FAId: string;
  published
    property ACol: Integer read FACol write FACol;
    property ARow: Integer read FARow write FARow;
    property ADataField: string read FADataField write FADataField;
    property AId: string read FAId write FAId;
    property AValue: string read FAValue write FAValue;
  end;

  TJQXCellEvent = procedure(Sender: TObject; Event: TJQXCellEventArgs) of object;

  TJQXCellDataEventArgs = class(TPersistent)
  private
    FARow: Integer;
    FACol: Integer;
    FADataField: string;
    FAValue: string;
    FAId: string;
    FAData: string;
  published
    property ACol: Integer read FACol write FACol;
    property ARow: Integer read FARow write FARow;
    property AData: string read FAData write FAData;
    property ADataField: string read FADataField write FADataField;
    property AId: string read FAId write FAId;
    property AValue: string read FAValue write FAValue;
  end;

  TJQXCellDataEvent = procedure(Sender: TObject; Event: TJQXCellDataEventArgs) of object;

  TJQXCellEditEventArgs = class(TPersistent)
  private
    FARow: Integer;
    FACol: Integer;
    FADataField: string;
    FAValue: string;
    FAId: string;
    FAOldValue: string;
  published
    property ACol: Integer read FACol write FACol;
    property ARow: Integer read FARow write FARow;
    property ADataField: string read FADataField write FADataField;
    property AId: string read FAId write FAId;
    property AValue: string read FAValue write FAValue;
    property AOldValue: string read FAOldValue write FAOldValue;
  end;

  TJQXCellEditEvent = procedure(Sender: TObject; Event: TJQXCellEditEventArgs) of object;

  TJQXCellValidateEventArgs = class(TPersistent)
  private
    FARow: Integer;
    FACol: Integer;
    FADataField: string;
    FAValue: string;
    FAId: string;
    FAOldValue: string;
    FAMessage: string;
    FAAllow: Boolean;
  published
    property ACol: Integer read FACol write FACol;
    property ARow: Integer read FARow write FARow;
    property ADataField: string read FADataField write FADataField;
    property AId: string read FAId write FAId;
    property AValue: string read FAValue write FAValue;
    property AOldValue: string read FAOldValue write FAOldValue;
    property AAllow: Boolean read FAAllow write FAAllow default true;
    property AMessage: string read FAMessage write FAMessage;
  end;

  TJQXCellValidateEvent = procedure(Sender: TObject; Event: TJQXCellValidateEventArgs) of object;

  TJQXPageEventArgs = class(TPersistent)
  private
    FAPageIndex: Integer;
    FAPageSize: Integer;
  published
    property APageIndex: Integer read FAPageIndex write FAPageIndex;
    property APageSize: Integer read FAPageSize write FAPageSize;
  end;

  TJQXPageEvent = procedure(Sender: TObject; Event: TJQXPageEventArgs) of object;

  TJQXRowEventArgs = class(TPersistent)
  private
    FARowIndex: Integer;
  published
    property ARowIndex: Integer read FARowIndex write FARowIndex;
  end;

  TJQXRowEvent = procedure(Sender: TObject; Event: TJQXRowEventArgs) of object;

  TJQXSortEventArgs = class(TPersistent)
  private
    FACol: Integer;
    FASortDirection: TJQXGridSortDirection;
    FADataField: string;
  published
    property ACol: Integer read FACol write FACol;
    property ADataField: string read FADataField write FADataField;
    property ASortDirection: TJQXGridSortDirection read FASortDirection write FASortDirection default gsdUnsorted;
  end;

  TJQXSortEvent = procedure(Sender: TObject; Event: TJQXSortEventArgs) of object;

  TJQXFilterEventArgs = class(TPersistent)
  private
    FACol: Integer;
    FADataField: string;
    FAValue: string;
    FACondition: string;
  published
    property ACol: Integer read FACol write FACol;
    property ADataField: string read FADataField write FADataField;
    property AValue: string read FAValue write FAValue;
    property ACondition: string read FACondition write FACondition;
  end;

  TJQXFilterEvent = procedure(Sender: TObject; Event: TJQXFilterEventArgs) of object;

  TWebJQXGrid = class;
  TJQXGridColumns = class;
  TJQXGridOptions = class;

  TJQXGridColumn = class(TCollectionItem)
  private
    FAlignment: TAlignment;
    FDataField: string;
    FEditor: TJQXGridEditor;
    FFormat: string;
    FTitle: string;
    FWidth: Integer;
    FOwner: TJQXGridColumns;
    FFreeze: Boolean;
    FColumnType: TJQXGridColumnType;
    FDataType: TJQXGridColumnDataType;
  public
    constructor Create(ACollection: TCollection);  override;
    destructor Destroy; override;
    procedure Assign(Source: TPersistent); override;
    function GetOwner: TPersistent; override;
  protected
    function GetDisplayName: string; override;
  published
    property Alignment: TAlignment read FAlignment write FAlignment default taLeftJustify;
    property ColumnType: TJQXGridColumnType read FColumnType write FColumnType default gctDefault;
    property DataField: string read FDataField write FDataField;
    property DataType: TJQXGridColumnDataType read FDataType write FDataType default gcdString;
    property Editor: TJQXGridEditor read FEditor write FEditor default geNone;
    property Freeze: Boolean read FFreeze write FFreeze default false;
    property Format: string read FFormat write FFormat;
    property Title: string read FTitle write FTitle;
    property Width: Integer read FWidth write FWidth default 0;
  end;

  TJQXGridColumns = class(TCollection)
  private
    FOwner: TWebJQXGrid;
    function GetItems(Index: Integer): TJQXGridColumn;
    procedure SetItems(Index: Integer; const Value: TJQXGridColumn);
  protected
    procedure Update(Item: TCollectionItem); override;
  public
    constructor Create(AOwner: TWebJQXGrid);
    function GetOwner: TPersistent; override;
    function Add: TJQXGridColumn;
    function Insert(Index: Integer):TJQXGridColumn;
    property Items[Index: Integer]: TJQXGridColumn read GetItems write SetItems; default;
  end;

  TJQXGridBands = class(TPersistent)
  private
    FOwner: TJQXGridOptions;
    FRowCount: Integer;
    FEnabled: Boolean;
  public
    constructor Create(AOwner: TJQXGridOptions); overload;
    destructor Destroy; override;
    procedure Assign(Source: TPersistent); override;
    function GetOwner: TPersistent; override;
  published
    property Enabled: Boolean read FEnabled write FEnabled default False;
    property RowCount: Integer read FRowCount write FRowCount default 1;
  end;

  TJQXGridPaging = class(TPersistent)
  private
    FOwner: TJQXGridOptions;
    FEnabled: Boolean;
    FPageSize: Integer;
  public
    constructor Create(AOwner: TJQXGridOptions); overload;
    destructor Destroy; override;
    procedure Assign(Source: TPersistent); override;
    function GetOwner: TPersistent; override;
  published
    property Enabled: Boolean read FEnabled write FEnabled default False;
    property PageSize: Integer read FPageSize write FPageSize default 10;
   end;

  TJQXGridSorting = class(TPersistent)
  private
    FOwner: TJQXGridOptions;
    FEnabled: Boolean;
    FDirection: TJQXGridSortDirection;
    FColumnIndex: Integer;
  public
    constructor Create(AOwner: TJQXGridOptions); overload;
    destructor Destroy; override;
    procedure Assign(Source: TPersistent); override;
    function GetOwner: TPersistent; override;
  published
    property Enabled: Boolean read FEnabled write FEnabled default False;
    property ColumnIndex: Integer read FColumnIndex write FColumnIndex default -1;
    property Direction: TJQXGridSortDirection read FDirection write FDirection default gsdAscending;
   end;

  TJQXGridOptions = class(TPersistent)
  private
    FOwner: TWebJQXGrid;
    FEditing: Boolean;
    FFiltering: Boolean;
    FSelectionMode: TJQXGridSelection;
    FGrouping: Boolean;
    FHovering: Boolean;
    FBands: TJQXGridBands;
    FPaging: TJQXGridPaging;
    FSorting: TJQXGridSorting;
  public
    constructor Create(AOwner: TWebJQXGrid);
    destructor Destroy; override;
    procedure Assign(Source: TPersistent); override;
    function GetOwner: TPersistent; override;
  published
    property Bands: TJQXGridBands read FBands write FBands;
    property Editing: Boolean read FEditing write FEditing default True;
    property Grouping: Boolean read FGrouping write FGrouping default False;
    property Hovering: Boolean read FHovering write FHovering default True;
    property Filtering: Boolean read FFiltering write FFiltering default False;
    property Paging: TJQXGridPaging read FPaging write FPaging;
    property Sorting: TJQXGridSorting read FSorting write FSorting;
    property SelectionMode: TJQXGridSelection read FSelectionMode write FSelectionMode default gsSingleRow;
  end;

  TJQXGridData = class(TPersistent)
  private
    FOwner: TWebJQXGrid;
    FDataDelimiter: char;
    FDataType: TJQXGridDataType;
    FDataUrl: string;
    FDataId: string;
    FDataJSON: TJSJSON;
    FDataArray: TJSArray;
  public
    constructor Create(AOwner: TWebJQXGrid); overload;
    destructor Destroy; override;
    procedure Assign(Source: TPersistent); override;
    function GetOwner: TPersistent; override;

    property JSON: TJSJSON read FDataJSON write FDataJSON;
    property DataArray: TJSArray read FDataArray write FDataArray;
//  published
    property DataType: TJQXGridDataType read FDataType write FDataType default gdtJSON;
    property Url: string read FDataUrl write FDataUrl;
    property Id: string read FDataId write FDataId;
    property Delimiter: char read FDataDelimiter write FDataDelimiter;
  end;

  TJQXGridCellRec = record
    Col, Row: Integer;
  end;

  TJQXGridExportState = (esExportStart, esExportNewRow, esExportDone, esExportSelRow, esExportFail, esExportNextRow, esExportFindRow);

  TWebJQXGridAdapter = class(TComponent)
  private
    FBlockAdd: Boolean;
    FGrid: TWebJQXGrid;
    FActive: Boolean;
    procedure SetActive(const Value: boolean);
    procedure SetGrid(const Value: TWebJQXGrid);
  protected
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    procedure UpdateBounds; virtual;
    procedure Initialize; virtual;
    procedure ScrollGrid; virtual;
    procedure SelectCell(ACell: TJQXGridCellRec); virtual;
    procedure CellEditValidateData(ACol, ARow: Integer; var CellString: String; var Allow: Boolean); virtual;
    procedure CellEditGetData(ACol, ARow: Integer; var CellString: String); virtual;
    procedure GetCellData(ACol, ARow: Integer; ADataField: string; var ACellData: string); virtual;
    procedure ExportNotification(AState: TJQXGridExportState; ARow: Integer); virtual;
  public
    constructor Create(AOwner: TComponent); override;
  published
    property Active: Boolean read FActive write SetActive default False;
    property Grid: TWebJQXGrid read FGrid write SetGrid;
  end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebJQXGrid = class(TWebJQXCustomControl)
  private
    FBlockAdd: Boolean;
    FColumns: TJQXGridColumns;
    FRowCount: Integer;
    FOptions: TJQXGridOptions;
    FOnCellEditDone: TJQXCellEditEvent;
    FOnCellClick: TJQXCellEvent;
    FRowHeight: Integer;
//    FScrollMode: TJQXGridScrollMode;
    FOnCellSelect: TJQXCellEvent;
    FOnPageChange: TJQXPageEvent;
    FOnRowSelect: TJQXRowEvent;
    FOnRowClick: TJQXRowEvent;
    FOnSort: TJQXSortEvent;
    FOnGetCellData: TJQXCellDataEvent;
    FOnCellEditStart: TJQXCellEvent;
    FOnCellEditValidate: TJQXCellValidateEvent;
    FOnFilter: TJQXFilterEvent;
    FData: TJQXGridData;
    FAdapter: TWebJQXGridAdapter;
    function GetCells(Col, Row: Integer): string;
    procedure SetCells(Col, Row: Integer; const Value: string);
    function GetRowSelect(Row: integer): boolean;
    procedure SetRowSelect(Row: integer; const Value: boolean);
    procedure SetAdapter(const Value: TWebJQXGridAdapter);
    function GetFocusedCell: TJQXGridCellRec;
    function GetVisibleRowCount: Integer;
    procedure SetFocusedCell(const Value: TJQXGridCellRec);
    procedure SetColumns(const Value: TJQXGridColumns);
    procedure SetData(const Value: TJQXGridData);
  protected
    procedure ExportNotification(AState: TJQXGridExportState; ARow: Integer); virtual;
    property Adapter: TWebJQXGridAdapter read FAdapter write SetAdapter;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    property Cells[Col,Row: Integer]: string read GetCells write SetCells;
    property RowSelect[Row: integer]: boolean read GetRowSelect write SetRowSelect;
    property Data: TJQXGridData read FData write SetData;

    procedure DeleteRow(Row: integer);
    procedure DeleteRows(Row,Count: integer); virtual;
    procedure InsertRow(Row: integer);
    procedure InsertRows(Row,Count: integer); virtual;

    procedure SelectCell(ACell: TJQXGridCellRec);
    procedure SelectRow(ARow: Integer);
    property VisibleRowCount: Integer read GetVisibleRowCount;
    property FocusedCell: TJQXGridCellRec read GetFocusedCell write SetFocusedCell;
  published
    property Columns: TJQXGridColumns read FColumns write SetColumns;
    property Enabled;
    property Options: TJQXGridOptions read FOptions write FOptions;
    property RowCount: Integer read FRowCount write FRowCount default 5;
    property RowHeight: Integer read FRowHeight write FRowHeight default 28;
    property Theme;
//    property ScrollMode: TJQXGridScrollMode read FScrollMode write FScrollMode default gsmPixelScrolling;

    property OnCellEditDone: TJQXCellEditEvent read FOnCellEditDone write FOnCellEditDone;
    property OnCellEditStart: TJQXCellEvent read FOnCellEditStart write FOnCellEditStart;
    property OnCellEditValidate: TJQXCellValidateEvent read FOnCellEditValidate write FOnCellEditValidate;
    property OnCellClick: TJQXCellEvent read FOnCellClick write FOnCellClick;
    property OnCellSelect: TJQXCellEvent read FOnCellSelect write FOnCellSelect;
    property OnGetCellData: TJQXCellDataEvent read FOnGetCellData write FOnGetCellData;
    property OnPageChange: TJQXPageEvent read FOnPageChange write FOnPageChange;
    property OnRowClick: TJQXRowEvent read FOnRowClick write FOnRowClick;
    property OnRowSelect: TJQXRowEvent read FOnRowSelect write FOnRowSelect;
    property OnSort: TJQXSortEvent read FOnSort write FOnSort;
    property OnFilter: TJQXFilterEvent read FOnFilter write FOnFilter;
  end;


function MakeCell(ACol,ARow: integer): TJQXGridCellRec;

implementation

{ TWebJQXGrid }

function MakeCell(ACol,ARow: integer): TJQXGridCellRec;
begin
  Result.Col := ACol;
  Result.Row := ARow;
end;

constructor TWebJQXGrid.Create(AOwner: TComponent);
var
  I: integer;
begin
  inherited;
  Width := 300;
  Height := 200;
  FColumns := TJQXGridColumns.Create(Self);
  FData := TJQXGridData.Create(Self);
  FOptions := TJQXGridOptions.Create(Self);
  FRowCount := 5;
  FRowHeight := 28;
//  FScrollMode := gsmPixelScrolling;

  for I := 0 to 4 do
    FColumns.Add;
end;

procedure TWebJQXGrid.DeleteRow(Row: integer);
begin

end;

procedure TWebJQXGrid.DeleteRows(Row, Count: integer);
begin

end;

destructor TWebJQXGrid.Destroy;
begin
  FColumns.Free;
  FData.Free;
  FOptions.Free;
  inherited;
end;

procedure TWebJQXGrid.ExportNotification(AState: TJQXGridExportState;
  ARow: Integer);
begin

end;

function TWebJQXGrid.GetCells(Col, Row: Integer): string;
begin
  Result := '';
end;

function TWebJQXGrid.GetFocusedCell: TJQXGridCellRec;
begin

end;

function TWebJQXGrid.GetRowSelect(Row: integer): boolean;
begin
  Result := False;
end;

function TWebJQXGrid.GetVisibleRowCount: Integer;
begin
  Result := 0;
end;

procedure TWebJQXGrid.InsertRow(Row: integer);
begin

end;

procedure TWebJQXGrid.InsertRows(Row, Count: integer);
begin

end;

procedure TWebJQXGrid.SelectCell(ACell: TJQXGridCellRec);
begin

end;

procedure TWebJQXGrid.SelectRow(ARow: Integer);
begin

end;

procedure TWebJQXGrid.SetAdapter(const Value: TWebJQXGridAdapter);
begin
  if FAdapter <> Value then
  begin
    FAdapter := Value;
    if FBlockAdd then
      Exit;

    FBlockAdd := True;
    if Assigned(FAdapter) then
    begin
      FAdapter.Grid := Self;
      FAdapter.Initialize;
    end;
    FBlockAdd := False;
  end;
end;

procedure TWebJQXGrid.SetCells(Col, Row: Integer; const Value: string);
begin
//
end;

procedure TWebJQXGrid.SetColumns(const Value: TJQXGridColumns);
begin
//
end;

procedure TWebJQXGrid.SetData(const Value: TJQXGridData);
begin
//
end;

procedure TWebJQXGrid.SetFocusedCell(const Value: TJQXGridCellRec);
begin
//
end;

procedure TWebJQXGrid.SetRowSelect(Row: integer; const Value: boolean);
begin
//
end;

{ TJQXGridColumn }

procedure TJQXGridColumn.Assign(Source: TPersistent);
begin
  if (Source is TJQXGridColumn) then
  begin
    FAlignment := (Source as TJQXGridColumn).Alignment;
    FColumnType := (Source as TJQXGridColumn).ColumnType;
    FDataField := (Source as TJQXGridColumn).DataField;
    FDataType := (Source as TJQXGridColumn).DataType;
    FEditor := (Source as TJQXGridColumn).Editor;
    FFreeze := (Source as TJQXGridColumn).Freeze;
    FFormat := (Source as TJQXGridColumn).Format;
    FTitle := (Source as TJQXGridColumn).Title;
    FWidth := (Source as TJQXGridColumn).Width;
  end;
end;

constructor TJQXGridColumn.Create(ACollection: TCollection);
begin
  inherited;
  FAlignment := taLeftJustify;
  FColumnType := gctDefault;
  FDataField := '';
  FDataType := gcdString;
  FEditor := geNone;
  FFreeze := false;
  FFormat := '';
  FTitle := '';
  FWidth := 100;

  if ACollection is TJQXGridColumns then
  begin
    FOwner := ACollection as TJQXGridColumns;
  end;
end;

destructor TJQXGridColumn.Destroy;
begin
  inherited;
end;

function TJQXGridColumn.GetDisplayName: string;
begin
  Result := 'Column '+ IntToStr(Index);
end;

function TJQXGridColumn.GetOwner: TPersistent;
begin
  Result := FOwner;
end;

{ TJQXGridColumns }

function TJQXGridColumns.Add: TJQXGridColumn;
begin
  Result := TJQXGridColumn(inherited Add);
end;

constructor TJQXGridColumns.Create(AOwner: TWebJQXGrid);
begin
  inherited Create(TJQXGridColumn);
  FOwner := AOwner;
end;

function TJQXGridColumns.GetItems(Index: Integer): TJQXGridColumn;
begin
  Result := TJQXGridColumn(inherited Items[Index]);
end;

function TJQXGridColumns.GetOwner: TPersistent;
begin
  Result := FOwner;
end;

function TJQXGridColumns.Insert(Index: Integer): TJQXGridColumn;
begin
  Result := TJQXGridColumn(inherited Insert(Index));
end;

procedure TJQXGridColumns.SetItems(Index: Integer; const Value: TJQXGridColumn);
begin
  inherited Items[Index] := Value;
end;

procedure TJQXGridColumns.Update(Item: TCollectionItem);
begin
  inherited;
end;

{ TJQXGridBands }

procedure TJQXGridBands.Assign(Source: TPersistent);
begin
  if (Source is TJQXGridBands) then
  begin
    FEnabled := (Source as TJQXGridBands).Enabled;
    FRowCount := (Source as TJQXGridBands).RowCount;
  end;
end;

constructor TJQXGridBands.Create(AOwner: TJQXGridOptions);
begin
  FEnabled := False;
  FRowCount := 1;
  FOwner := AOwner;
end;

destructor TJQXGridBands.Destroy;
begin
  inherited;
end;

function TJQXGridBands.GetOwner: TPersistent;
begin
  Result := FOwner;
end;

{ TJQXGridPaging }

procedure TJQXGridPaging.Assign(Source: TPersistent);
begin
  if (Source is TJQXGridPaging) then
  begin
    FEnabled := (Source as TJQXGridPaging).Enabled;
    FPageSize := (Source as TJQXGridPaging).PageSize;
  end;
end;

constructor TJQXGridPaging.Create(AOwner: TJQXGridOptions);
begin
  FEnabled := False;
  FPageSize := 10;
  FOwner := AOwner;
end;

destructor TJQXGridPaging.Destroy;
begin
  inherited;
end;

function TJQXGridPaging.GetOwner: TPersistent;
begin
  Result := FOwner;
end;

{ TJQXGridSorting }

procedure TJQXGridSorting.Assign(Source: TPersistent);
begin
  inherited;
  if (Source is TJQXGridSorting) then
  begin
    FEnabled := (Source as TJQXGridSorting).Enabled;
    FColumnIndex := (Source as TJQXGridSorting).ColumnIndex;
    FDirection := (Source as TJQXGridSorting).Direction;
  end;
end;

constructor TJQXGridSorting.Create(AOwner: TJQXGridOptions);
begin
  FEnabled := False;
  FColumnIndex := -1;
  FDirection := gsdAscending;
  FOwner := AOwner;
end;

destructor TJQXGridSorting.Destroy;
begin
  inherited;
end;

function TJQXGridSorting.GetOwner: TPersistent;
begin
  Result := FOwner;
end;

{ TJQXGridData }

procedure TJQXGridData.Assign(Source: TPersistent);
begin
//  if (Source is TJQXGridData) then
//  begin
//    FDataType := (Source as TJQXGridData).DataType;
//    FDataUrl := (Source as TJQXGridData).Url;
//    FDataJSON := (Source as TJQXGridData).JSON;
//    FDataArray := (Source as TJQXGridData).DataArray;
//    FDataId := (Source as TJQXGridData).Id;
//    FDataDelimiter := (Source as TJQXGridData).Delimiter;
//  end;
end;

constructor TJQXGridData.Create(AOwner: TWebJQXGrid);
begin
  FDataType := gdtNone;
  FDataUrl := '';
  FDataJSON := nil;
  FDataArray := nil;
  FDataId := '';
  FDataDelimiter := ',';
  FOwner := AOwner;
end;

destructor TJQXGridData.Destroy;
begin
  inherited;
end;

function TJQXGridData.GetOwner: TPersistent;
begin
  Result := FOwner;
end;


{ TJQXGridOptions }

procedure TJQXGridOptions.Assign(Source: TPersistent);
begin
  if (Source is TJQXGridOptions) then
  begin
    FBands := (Source as TJQXGridOptions).Bands;
    FEditing := (Source as TJQXGridOptions).Editing;
    FGrouping := (Source as TJQXGridOptions).Grouping;
    FHovering := (Source as TJQXGridOptions).Hovering;
    FFiltering := (Source as TJQXGridOptions).Filtering;
    FPaging := (Source as TJQXGridOptions).Paging;
    FSelectionMode := (Source as TJQXGridOptions).SelectionMode;
    FSorting := (Source as TJQXGridOptions).Sorting;
  end;
end;

constructor TJQXGridOptions.Create(AOwner: TWebJQXGrid);
begin
  inherited Create;
  FBands := TJQXGridBands.Create(Self);
  FEditing := True;
  FGrouping := False;
  FHovering := True;
  FFiltering := False;
  FPaging := TJQXGridPaging.Create(Self);
  FSelectionMode := gsSingleRow;
  FSorting := TJQXGridSorting.Create(Self);
  FOwner := AOwner;
end;

destructor TJQXGridOptions.Destroy;
begin
  FBands.Free;
  FPaging.Free;
  FSorting.Free;
  inherited;
end;

function TJQXGridOptions.GetOwner: TPersistent;
begin
  Result := FOwner;
end;

{ TWebJQXGridAdapter }

procedure TWebJQXGridAdapter.CellEditGetData(ACol, ARow: Integer;
  var CellString: String);
begin

end;

procedure TWebJQXGridAdapter.CellEditValidateData(ACol, ARow: Integer;
  var CellString: String; var Allow: Boolean);
begin

end;

constructor TWebJQXGridAdapter.Create(AOwner: TComponent);
var
  I: Integer;
begin
  inherited;
  FActive := False;
  if Assigned(AOwner) and (AOwner is TCustomForm) then
  begin
    for I := 0 to AOwner.ComponentCount - 1 do
    begin
      if (AOwner.Components[i] is TWebJQXGrid) then
      begin
        Grid := AOwner.Components[i] as TWebJQXGrid;
        Break;
      end;
    end;
  end;
end;

procedure TWebJQXGridAdapter.ExportNotification(AState: TJQXGridExportState;
  ARow: Integer);
begin

end;

procedure TWebJQXGridAdapter.GetCellData(ACol, ARow: Integer;
  ADataField: string; var ACellData: string);
begin

end;

procedure TWebJQXGridAdapter.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited;
  if (Operation = opRemove) and (AComponent = FGrid) then
    FGrid := nil;
end;

procedure TWebJQXGridAdapter.ScrollGrid;
begin

end;

procedure TWebJQXGridAdapter.SelectCell(ACell: TJQXGridCellRec);
begin

end;

procedure TWebJQXGridAdapter.SetActive(const Value: boolean);
begin
  FActive := Value;
  if not Assigned(Grid) then
    Exit;
  Initialize;
end;

procedure TWebJQXGridAdapter.SetGrid(const Value: TWebJQXGrid);
begin
  if FGrid <> Value then
  begin
    FGrid := Value;
    if FBlockAdd then
      Exit;

    FBlockAdd := True;
    if Assigned(FGrid) then
    begin
      FGrid.Adapter := Self;
      Initialize;
    end;
    FBlockAdd := False;
  end;
end;

procedure TWebJQXGridAdapter.UpdateBounds;
begin

end;

procedure TWebJQXGridAdapter.Initialize;
begin
end;

end.
