{********************************************************************}
{                                                                    }
{ written by TMS Software                                            }
{            copyright (c) 2018                                      }
{            Email : info@tmssoftware.com                            }
{            Web : http://www.tmssoftware.com                        }
{                                                                    }
{ The source code is given as is. The author is not responsible      }
{ for any possible damage done due to the use of this code.          }
{ The complete source code remains property of the author and may    }
{ not be distributed, published, given or sold in any form as such.  }
{ No parts of the source code can be included in any other component }
{ or application without written authorization of the author.        }
{********************************************************************}
unit WEBLib.PayPal;

interface

uses
  {$IFNDEF LCLTMSWEB}
  System.Classes, WinApi.Windows, System.SysUtils, Vcl.Controls, Vcl.Graphics,
  Types, WEBLib.Controls;
  {$ENDIF}
  {$IFDEF LCLTMSWEB}
  Classes, SysUtils, Controls, Graphics, Types, WEBLib.Controls;
  {$ENDIF}

type
  TPayPalPaymentEventArgs = class(TPersistent)
  private
    FPayerID: string;
    FPaymentID: string;
    FOrderID: string;
    FState: string;
    FEmail: string;
    FCustomText: string;
    FInvoiceNumber: string;
    FLastName: string;
    FFirstName: string;
    FPostalCode: string;
    FPaymentState: string;
    FCity: string;
    FCountryCode: string;
    FPhone: string;
    FAddress2: string;
    FAddress1: string;
    FRecipientName: string;
    FTotal: string;
    FSaleID: string;
    FDescription: string;
    FCurrency: string;
  public
    property Description: string read FDescription write FDescription;
    property PaymentID: string read FPaymentID write FPaymentID;
    property PayerID: string read FPayerID write FPayerID;
    property OrderID: string read FOrderID write FOrderID;
    property SaleID: string read FSaleID write FSaleID;
    property PaymentState: string read FPaymentState write FPaymentState;
    property Email: string read FEmail write FEmail;
    property Phone: string read FPhone write FPhone;
    property CustomText: string read FCustomText write FCustomText;
    property InvoiceNumber: string read FInvoiceNumber write FInvoiceNumber;
    property FirstName: string read FFirstName write FFirstName;
    property LastName: string read FLastName write FLastName;
    property Address1: string read FAddress1 write FAddress1;
    property Address2: string read FAddress2 write FAddress2;
    property City: string read FCity write FCity;
    property PostalCode: string read FPostalCode write FPostalCode;
    property CountryCode: string read FCountryCode write FCountryCode;
    property State: string read FState write FState;
    property RecipientName: string read FRecipientName write FRecipientName;
    property Total: string read FTotal write FTotal;
    property Currency: string read FCurrency write FCurrency;
  end;

  TPayPalPaymentEvent = procedure(Sender: TObject; Args: TPayPalPaymentEventArgs) of object;

  TPayPalErrorEventArgs = class(TPersistent)
  private
    FErrorDetails: TStringList;
    FErrorName: string;
  public
    property ErrorName: string read FErrorName write FErrorName;
    property ErrorDetails: TStringList read FErrorDetails write FErrorDetails;
  end;

  TPayPalErrorEvent = procedure(Sender: TObject; Args: TPayPalErrorEventArgs) of object;

  TPayPalCurrency = (pcAUD, pcBRL, pcCAD, pcCZK, pcDKK, pcEUR, pcHKD, pcHUF,
    pcILS, pcJPY, pcMYR, pcMXN, pcTWD, pcNZD, pcNOK, pcPHP, pcPLN, pcGBP, pcRUB,
    pcSGD, pcSEK, pcCHF, pcTHB, pcUSD);
  TPayPalLocale = (plEN_US, plDE_DE, plNL_NL, plFR_FR, plPT_BR, plFR_CA, plZH_CN,
    plDA_DK, plRU_RU, plIT_IT, plNO_NO, plPL_PL, plPT_PT, plES_ES, plSV_SE,
    plEN_GB, plJA_JP);

  TPayPalPayment = class;
  TPayPalItems = class;

  TPayPalItem = class(TCollectionItem)
  private
    FName: string;
    FPrice: Double;
    FQuantity: Integer;
    FDescription: string;
    FSKU: string;
    FTax: Double;
    FTag: Integer;
    FTagObject: TObject;
  public
    constructor Create(ACollection: TCollection); override;
    destructor Destroy; override;
  published
    property Name: string read FName write FName;
    property Price: Double read FPrice write FPrice;
    property Quantity: Integer read FQuantity write FQuantity default 1;
    property Description: string read FDescription write FDescription;
    property Tax: Double read FTax write FTax;
    property SKU: String read FSKU write FSKU;
    property Tag: Integer read FTag write FTag default 0;
    property TagObject: TObject read FTagObject write FTagObject;
  end;

  TPayPalItems = class(TCollection)
  private
    FOwner: TPayPalPayment;
    function GetItems(Index: integer): TPayPalItem;
    procedure SetItems(Index: integer; const Value: TPayPalItem);
  public
    constructor Create(AOwner: TPayPalPayment); overload;
    function GetOwner: TPersistent; override;
    function Add: TPayPalItem; reintroduce;
    function Insert(Index: Integer): TPayPalItem; reintroduce;
    property Items[Index: integer]: TPayPalItem read GetItems write SetItems; default;
  end;

  TPayPalPayment = class(TPersistent)
  private
    FInsurance: Double;
    FTax: Double;
    FShipping: Double;
    FInvoiceNumber: string;
    FDescription: string;
    FCustomText: string;
    FCurrency: TPayPalCurrency;
    FLocale: TPayPalLocale;
    FItems: TPayPalItems;
    FShippingDiscount: Double;
    FHandlingFee: Double;
    FPostalCode: string;
    FState: string;
    FPhone: string;
    FAddress2: string;
    FAddress1: string;
    FRecipientName: string;
    FCity: string;
    FCountryCode: string;
  public
    constructor Create; reintroduce;
    destructor Destroy; override;
    procedure Assign(Source: TPersistent); override;
  published
    property Tax: Double read FTax write FTax;
    property Shipping: Double read FShipping write FShipping;
    property ShippingDiscount: Double read FShippingDiscount write FShippingDiscount;
    property HandlingFee: Double read FHandlingFee write FHandlingFee;
    property Insurance: Double read FInsurance write FInsurance;
    property CustomText: string read FCustomText write FCustomText;
    property InvoiceNumber: string read FInvoiceNumber write FInvoiceNumber;
    property Description: string read FDescription write FDescription;
    property Currency: TPayPalCurrency read FCurrency write FCurrency default pcUSD;
    property Locale: TPayPalLocale read FLocale write FLocale default plEN_US;
    property Items: TPayPalItems read FItems write FItems;
    property Address1: string read FAddress1 write FAddress1;
    property Address2: string read FAddress2 write FAddress2;
    property City: string read FCity write FCity;
    property PostalCode: string read FPostalCode write FPostalCode;
    property CountryCode: string read FCountryCode write FCountryCode;
    property State: string read FState write FState;
    property Phone: string read FPhone write FPhone;
    property RecipientName: string read FRecipientName write FRecipientName;
  end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebPayPal = class(TWebCustomControl)
  private
    FAPIKey: string;
    FOnPaymentCancelled: TNotifyEvent;
    FOnPaymentError: TPayPalErrorEvent;
    FOnPaymentDone: TPayPalPaymentEvent;
    FPayment: TPayPalPayment;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property Enabled;
    property ElementClassName;
    property ElementID;
    property Visible;
    property Width;
    property Height;

    property APIKey: string read FAPIKey write FAPIKey;
    property Payment: TPayPalPayment read FPayment write FPayment;

    property OnPaymentDone: TPayPalPaymentEvent read FOnPaymentDone write FOnPaymentDone;
    property OnPaymentCancelled: TNotifyEvent read FOnPaymentCancelled write FOnPaymentCancelled;
    property OnPaymentError: TPayPalErrorEvent read FOnPaymentError write FOnPaymentError;
  end;


implementation

{ TWebPayPal }

constructor TWebPayPal.Create(AOwner: TComponent);
begin
  inherited;
  Width := 200;
  Height := 40;
  FAPIKey := '';
  FPayment := TPayPalPayment.Create;
end;

destructor TWebPayPal.Destroy;
begin
  FPayment.Free;
  inherited;
end;

{ TPayPalPayment }

procedure TPayPalPayment.Assign(Source: TPersistent);
begin
  inherited;
  if Source is TPayPalPayment then
  begin
    FTax := (Source as TPayPalPayment).Tax;
    FShipping := (Source as TPayPalPayment).Shipping;
    FInsurance := (Source as TPayPalPayment).Insurance;
    FDescription := (Source as TPayPalPayment).Description;
    FCustomText := (Source as TPayPalPayment).CustomText;
    FInvoiceNumber := (Source as TPayPalPayment).InvoiceNumber;
    FCurrency := (Source as TPayPalPayment).Currency;
    FLocale := (Source as TPayPalPayment).Locale;
    FItems.Assign((Source as TPayPalPayment).Items);
  end;
end;

constructor TPayPalPayment.Create;
begin
  FTax := 0;
  FShipping := 0;
  FInsurance := 0;
  FDescription := '';
  FCustomText := '';
  FInvoiceNumber := '';
  FCurrency := pcUSD;
  FLocale := plEN_US;
  FItems := TPayPalItems.Create(Self);
end;

destructor TPayPalPayment.Destroy;
begin
  FItems.Free;
  inherited;
end;

{ TPayPalItem }

constructor TPayPalItem.Create(ACollection: TCollection);
begin
  inherited;
  FName := '';
  FPrice := 0;
  FQuantity := 1;
  FDescription := '';
  FTax := 0;
  FSKU := '';
  FTag := 0;
end;

destructor TPayPalItem.Destroy;
begin
  inherited;
end;

{ TPayPalItems }

function TPayPalItems.Add: TPayPalItem;
begin
  Result := TPayPalItem(inherited Add);
end;

constructor TPayPalItems.Create(AOwner: TPayPalPayment);
begin
  inherited Create(TPayPalItem);
  FOwner := AOwner;
end;

function TPayPalItems.GetItems(Index: integer): TPayPalItem;
begin
  Result := TPayPalItem(inherited Items[Index]);
end;

function TPayPalItems.GetOwner: TPersistent;
begin
  Result := FOwner;
end;

function TPayPalItems.Insert(Index: Integer): TPayPalItem;
begin
  Result := TPayPalItem(inherited Insert(Index));
end;

procedure TPayPalItems.SetItems(Index: integer; const Value: TPayPalItem);
begin
  inherited Items[Index] := Value;
end;

end.
