unit libelectron;

interface

uses
  JS, Types, Web, jsdelphisystem;

type
  TBrowserWindow = class;
  TWebContentsEventHandler = reference to procedure(event: TJSEvent; args: JSValue);
  TWebContentsEventArgsHandler = reference to procedure(event: TJSEvent; argf: JSValue; args: JSValue);
  TTrayIconClickEventHandler = reference to procedure;

  TJSElectronStringDynArray = array of string;

  TWebContents = class(TJSObject)
  private
    FId: Integer;
  public
    procedure closeDevTools;
    procedure copy;
    procedure copyImageAt(X, Y: NativeInt);
    procedure cut;
    procedure delete;
    procedure downloadURL(aURL: string);
    procedure focus;
    procedure goBack;
    procedure goForward;
    function insertText(Text: string): TJSPromise;
    function loadFile(aFilePath: string): TJSPromise;
    function loadURL(aURL: string): TJSPromise;
    procedure &on(aChannel: string; aListener: TWebContentsEventHandler); overload;
    procedure &on(aChannel: string; aListener: TWebContentsEventArgsHandler); overload;
    procedure openDevTools;
    procedure paste;
    procedure redo;
    procedure reload;
    procedure replace(Text: string);
    procedure replaceMisspelling(Text: string);
    procedure selectAll;
    procedure send(aChannel: string); overload;
    procedure send(aChannel: string; aArgs: JSValue); overload;
    procedure send(aChannel: string; aArgf: JSValue; aArgs: JSValue); overload;
    procedure startDrag(aItem: TJSObject);
    procedure stop;
    procedure undo;
    procedure unselect;
    procedure toggleDevTools;
    function getURL: string;
    function getTitle: string;
    function isDestroyed: Boolean;
    function isFocused: Boolean;
    function isLoading: Boolean;
    property id: Integer read FId write FId;
  end;

  TBrowserWindow = class(TJSObject)
  private
    FId: Integer;
    FWebContents: TWebContents;
    FFullScreenable: Boolean;
    FResizable: Boolean;
  public
    procedure blur;
    procedure close;
    procedure destroy; reintroduce;
    procedure focus;
    procedure hide;
    function loadURL(aURL: string): TJSPromise;
    function loadFile(aFilePath: string): TJSPromise;
    procedure setAlwaysOnTop(aFlag: Boolean);
    procedure setContentSize(aWidth: NativeInt; aHeight: NativeInt);
    procedure setEnabled(aEnable: Boolean);
    procedure setFullScreen(aFlag: Boolean);
    procedure setKiosk(aFlag: Boolean);
    procedure setMaximumSize(aWidth: NativeInt; aHeight: NativeInt);
    procedure setMinimumSize(aWidth: NativeInt; aHeight: NativeInt);
    procedure setSize(aWidth: NativeInt; aHeight: NativeInt);
    procedure setSkipTaskbar(aSkip: Boolean);
    procedure setTitle(aTitle: string);
    procedure show;
    function fromId(aId: NativeInt): TBrowserWindow;
    function getAllWindows: TJSObjectDynArray;
    function getFocusedWindow: TBrowserWindow;
    function getMaximumSize: TJSArray;
    function getMinimumSize: TJSArray;
    function getParentWindow: TBrowserWindow;
    function getSize: TJSArray;
    function getTitle: string;
    function isAlwaysOnTop: Boolean;
    function isFocused: Boolean;
    function isFullScreen: Boolean;
    function isKiosk: Boolean;
    function isVisible: Boolean;
    property fullScreenable: Boolean read FFullScreenable write FFullScreenable;
    property id: Integer read FId write FId;
    property resizable: Boolean read FResizable write FResizable;
    property webContents: TWebContents read FWebContents write FWebContents;
  end;

  TIPCEvent = class(TJSObject)
  private
    FReturnValue: JSValue;
    FSender: TWebContents;
  public
    property returnValue: JSValue read FReturnValue write FReturnValue;
    property sender: TWebContents read FSender;
  end;

  TIPCEventHandler = reference to procedure(event: TIPCEvent; args: JSValue);
  TIPCEventArgsHandler = reference to procedure(event: TIPCEvent; argf: JSValue; args: JSValue);

  TIPCMain = class
    procedure &on(aChannel: string; aListener: TIPCEventHandler); overload;
    procedure &on(aChannel: string; aListener: TIPCEventArgsHandler); overload;
    procedure once(aChannel: string; aListener: TIPCEventHandler); overload;
    procedure once(aChannel: string; aListener: TIPCEventArgsHandler); overload;
    procedure removeAllListeners; overload;
    procedure removeAllListeners(aChannel: string); overload;
    procedure removeListener(aChannel: string; aListener: TIPCEventHandler); overload;
    procedure removeListener(aChannel: string; aListener: TIPCEventArgsHandler); overload;
  end;

  TIPCRenderer = class
    function invoke(aChannel: string): TJSPromise; overload;
    function invoke(aChannel: string; aArgs: JSValue): TJSPromise; overload;
    procedure &on(aChannel: string; aListener: TIPCEventHandler); overload;
    procedure &on(aChannel: string; aListener: TIPCEventArgsHandler); overload;
    procedure once(aChannel: string; aListener: TIPCEventHandler); overload;
    procedure once(aChannel: string; aListener: TIPCEventArgsHandler); overload;
    procedure removeAllListeners(aChannel: string);
    procedure removeListener(aChannel: string; aListener: TIPCEventHandler); overload;
    procedure removeListener(aChannel: string; aListener: TIPCEventArgsHandler); overload;
    procedure send(aChannel: string); overload;
    procedure send(aChannel: string; aArgs: JSValue); overload;
    procedure send(aChannel: string; aArgf: JSValue; aArgs: JSValue); overload;
    procedure sendTo(aWebContentsID: NativeInt; aChannel: string); overload;
    procedure sendTo(aWebContentsID: NativeInt; aChannel: string; aArgs: JSValue); overload;
  end;

  TSelectOpenFileCallBack = reference to procedure (fileNames: TJSElectronStringDynArray);
  TSelectSaveFileCallBack = reference to procedure (fileName: String);
  TSelectMessageBoxCallBack = reference to procedure (response: NativeInt; checkboxChecked: Boolean);

  TDialog = class(TJSObject)
    procedure showErrorBox(aTitle: string; aContent: string);
    function showMessageBox(aBrowserWindow: TBrowserWindow; aOptions: TJSObject): TJSPromise; overload;
    function showMessageBox(aOptions: TJSObject): TJSPromise; overload;
    function showMessageBoxSync(aBrowserWindow: TBrowserWindow; aOptions: TJSObject): NativeInt; overload;
    function showMessageBoxSync(aOptions: TJSObject): NativeInt; overload;
    function showOpenDialog(aBrowserWindow: TBrowserWindow; aOptions: TJSObject): TJSPromise; overload;
    function showOpenDialog(aOptions: TJSObject): TJSPromise; overload;
    function showOpenDialogSync(aBrowserWindow: TBrowserWindow; aOptions: TJSObject): TJSElectronStringDynArray; overload;
    function showOpenDialogSync(aOptions: TJSObject): TJSElectronStringDynArray; overload;
    function showSaveDialog(aBrowserWindow: TBrowserWindow; aOptions: TJSObject): TJSPromise; overload;
    function showSaveDialog(aOptions: TJSObject): TJSPromise; overload;
    function showSaveDialogSync(aBrowserWindow: TBrowserWindow; aOptions: TJSObject): string; overload;
    function showSaveDialogSync(aOptions: TJSObject): string; overload;
  end;

  TNativeImageClass = class(TJSObject)
    function toDataURL: string;
  end;

  TNativeImage = class(TJSObject)
    function createEmpty: TNativeImageClass;
    function createFromDataURL(aDataURL: string): TNativeImageClass;
    function createFromPath(aPath: string): TNativeImageClass;
  end;

  TClipboard = class(TJSObject)
    procedure clear;
    procedure writeHTML(aMarkup: string);
    procedure writeImage(aImage: TNativeImageClass);
    procedure writeRTF(aText: string);
    procedure writeText(aText: string);
    function availableFormats: TJSElectronStringDynArray;
    function readText: string;
    function readHTML: string;
    function readImage: TNativeImageClass;
    function readRTF: string;
  end;

  TShell = class(TJSObject)
    procedure beep;
    function moveItemToTrash(aFullPath: string): Boolean;
    function openExternal(aURL: string): Boolean;
    function openItem(aFullPath: string): Boolean;
    function showItemInFolder(aFullPath: string): Boolean;
  end;

  TMenuClass = class;

  TMenuItemClass = class(TJSObject)
  private
    FChecked: Boolean;
    FEnabled: Boolean;
    FId: string;
    FLabel: string;
    FRole: string;
    FSubmenu: TMenuClass;
    FType: string;
    FVisible: Boolean;
  public
    procedure addEventListener(aName: string; aListener: JSValue);
    procedure click(aMenuItem: TMenuItemClass; aBrowserWindow: TBrowserWindow; aEvent: TJSEvent);
    property Checked: Boolean read FChecked write FChecked;
    property Enabled: Boolean read FEnabled write FEnabled;
    property Id: string read FId write FId;
    property &Label: string read FLabel write FLabel;
    property Role: string read FRole;
    property Submenu: TMenuClass read FSubmenu write FSubmenu;
    property &Type: string read FType;
    property Visible: Boolean read FVisible write FVisible;
  end;

  TMenuClass = class(TJSObject)
    procedure append(aMenuItem: TMenuItemClass);
    procedure closePopup; overload;
    procedure closePopup(aBrowserWindow: TBrowserWindow); overload;
    procedure popup(aOptions: TJSObject);
    function getMenuItemById(aId: string): TMenuItemClass;
  end;

  TMenu = class(TJSObject)
    procedure setApplicationMenu(aMenu: TMenuClass);
    function getApplicationMenu: TMenuClass;
  end;

  TTrayIcon = class(TJSObject)
    procedure destroy; reintroduce;
    procedure &on(aChannel: string; aListener: TTrayIconClickEventHandler);
    procedure setContextMenu(aMenu: TMenuClass);
    procedure setImage(aImage: TNativeImageClass);
    procedure setToolTip(aToolTip: string);
  end;

  TApplication = class(TJSObject)
    procedure quit;
    function getAppPath: string;
    function getPath(aName: string): string;
    function getName: string; //current application's name
    function getLocale: string;
    function isReady: Boolean;
  end;

  TGlobalShortcutCallback = reference to procedure;

  TGlobalShortcut = class(TJSObject)
    procedure &register(aAccelerator: string; aCallback: TGlobalShortcutCallback);
    procedure unregister(aAccelerator: string);
    procedure unregisterAll;
    function isRegistered(aAccelerator: string): Boolean;
  end;

  TApp = class(TJSObject)
  private
    FApplication: TApplication;
    FBrowserWindow: TBrowserWindow;
    FClipboard: TClipboard;
    FDialog: TDialog;
    FGlobalShortcut: TGlobalShortcut;
    FIPCMain: TIPCMain;
    FMenu: TMenu;
    FMenuItem: TMenuItemClass;
    FNativeImage: TNativeImage;
    FShell: TShell;
    FTrayIcon: TTrayIcon;
  public
    function getCurrentWindow: TBrowserWindow;
    function getCurrentWebContents: TWebContents;
    property Application: TApplication read FApplication;
    property BrowserWindow: TBrowserWindow read FBrowserWindow;
    property Clipboard: TClipboard read FClipboard;
    property Dialog: TDialog read FDialog;
    property GlobalShortcut: TGlobalShortcut read FGlobalShortcut;
    property IPCMain: TIPCMain read FIPCMain;
    property Menu: TMenu read FMenu;
    property MenuItem: TMenuItemClass read FMenuItem;
    property NativeImage: TNativeImage read FNativeImage;
    property Shell: TShell read FShell;
    property TrayIcon: TTrayIcon read FTrayIcon;
  end;

  TElectron = class(TJSObject)
  private
    FApp: TApp;
    FIPCRenderer: TIPCRenderer;
    FClipboard: TClipboard;
    FNativeImage: TNativeImage;
    FShell: TShell;
  public
    property App: TApp read FApp;
    property Clipboard: TClipboard read FClipboard;
    property IPCRenderer: TIPCRenderer read FIPCRenderer;
    property NativeImage: TNativeImage read FNativeImage;
    property Shell: TShell read FShell;
  end;

  TFileFilter = class(TJSObject)
  private
    FExtensions: TJSElectronStringDynArray;
    FName: string;
  public
    property Extensions: TJSElectronStringDynArray read FExtensions write FExtensions;
    property Name: string read FName write FName;
  end;

  TFileFilterArray = array of TFileFilter;

  TOpenDialogOptions = class(TJSObject)
  private
    FButtonLabel: string;
    FDefaultPath: string;
    FFilters: TFileFilterArray;
    FProperties: TJSElectronStringDynArray;
    FTitle: string;
  public
    property ButtonLabel: string read FButtonLabel write FButtonLabel;
    property DefaultPath: string read FDefaultPath write FDefaultPath;
    property Filters: TFileFilterArray read FFilters write FFilters;
    property Properties: TJSElectronStringDynArray read FProperties write FProperties;
    property Title: string read FTitle write FTitle;
  end;

  TSaveDialogOptions = class(TJSObject)
  private
    FButtonLabel: string;
    FDefaultPath: string;
    FFilters: TFileFilterArray;
    FTitle: string;
  public
    property ButtonLabel: string read FButtonLabel write FButtonLabel;
    property DefaultPath: string read FDefaultPath write FDefaultPath;
    property Filters: TFileFilterArray read FFilters write FFilters;
    property Title: string read FTitle write FTitle;
  end;

  TMessageBoxOptions = class(TJSObject)
  private
    FButtons: TJSElectronStringDynArray;
    FCancelId: NativeInt;
    FCheckboxChecked: Boolean;
    FCheckboxLabel: string;
    FDefaultId: NativeInt;
    FDetail: string;
    FIcon: TNativeImageClass;
    FMessage: string;
    FNoLink: Boolean;
    FNormalizeAccessKeys: Boolean;
    FTitle: string;
    FType: string;
  public
    property Buttons: TJSElectronStringDynArray read FButtons write FButtons;
    property CancelId: NativeInt read FCancelId write FCancelId;
    property CheckboxChecked: Boolean read FCheckboxChecked write FCheckboxChecked;
    property CheckboxLabel: string read FCheckboxLabel write FCheckboxLabel;
    property DefaultId: NativeInt read FDefaultId write FDefaultId;
    property Detail: string read FDetail write FDetail;
    property Icon: TNativeImageClass read FIcon write FIcon;
    property &Message: string read FMessage write FMessage;
    property NoLink: Boolean read FNoLink write FNoLink;
    property NormalizeAccessKeys: Boolean read FNormalizeAccessKeys write FNormalizeAccessKeys;
    property Title: string read FTitle write FTitle;
    property &Type: string read FType write FType;
  end;

  TJSElectronFile = class(TJSBlob)
  private
    FLastModified: NativeInt;
    FLastModifiedDate: TJSDate;
    FName: string;
    FPath: string;
    FType: string;
  public
    property lastModified: NativeInt read FLastModified;
    property lastModifiedDate: TJSDate read FLastModifiedDate;
    property name: string read FName;
    property path: string read FPath;
    property &type: string read FType;
  end;

  TJSElectronFileList = class(TJSEventTarget)
  private
    FLength: NativeInt;
  public
    function item(aIndex: NativeInt): TJSElectronFile;
    property length: NativeInt read FLength;
    property Files[aIndex: NativeInt]: TJSElectronFile read item; default;
  end;

  TJSNotificationPermissionCallBack = procedure(APermission: string) of object;
  TJSNotificationClickEvent = reference to procedure(AEvent: TJSEvent);

  TJSElectronNotification = class
    onshow: TJSEventHandler;
    onclick: TJSNotificationClickEvent;
    onclose: TJSEventHandler;
    constructor new(aTitle: string); overload;
    constructor new(aTitle: string; AOptions: TJSObject); overload;
    class function requestPermission: TJSPromise; overload;
    class procedure requestPermission(ACallBack: TJSNotificationPermissionCallBack); overload;
    procedure close;
  end;

  TNodeStat = class(TJSObject)
  protected
    FMTime: TJSDate;
  public
    property mtime: TJSDate read FMtime;
  end;

//  TReadFileCallBack = reference to procedure(err: TJSError; data: string);
//  TWriteFileCallBack = reference to procedure(err: TJSError);
//  TUnlinkCallBack = reference to procedure(err: TJSError);
  TWatchFileListener = reference to procedure(current: TNodeStat; previous: TNodeStat);

  TNodeFS = class(TJSObject)
//    procedure readFile(aPath: string; aCallBack: TReadFileCallBack);
//    procedure unlink(aPath: string; aCallback: TUnlinkCallBack);
    procedure unwatchFile(aFileName: string; aListener: TWatchFileListener);
    procedure watchFile(aFileName: string; aListener: TWatchFileListener);
//    procedure writeFile(aPath: string; aData: string; aCallBack: TWriteFileCallBack);
    procedure writeFileSync(aPath: string; aData: string);
    function existSync(aPath: string): Boolean;
    function readFileSync(aPath: string): TJSObject;
  end;

  TMenuKeyCap = (mkcBkSp, mkcTab, mkcEsc, mkcEnter, mkcSpace, mkcPgUp,
    mkcPgDn, mkcEnd, mkcHome, mkcLeft, mkcUp, mkcRight, mkcDown, mkcIns,
    mkcDel, mkcShift, mkcCtrl, mkcAlt);

var
  MenuKeyCaps: array[TMenuKeyCap] of string = (
    'Backspace', 'Tab', 'Escape', 'Return', 'Space', 'PageUp',
    'PageDown', 'End', 'Home', 'Left', 'Up', 'Right',
    'Down', 'Insert', 'Delete', 'Shift', 'CommandOrControl', 'Alt');

const
  scCommand = $1000;
  scShift = $2000;
  scCtrl = $4000;
  scAlt = $8000;
  scNone = 0;

function Electron: TElectron;
function FS: TNodeFS;

implementation
{$HINTS OFF}
{$WARNINGS OFF}

{ TWebContents }

procedure TWebContents.closeDevTools;
begin
end;

procedure TWebContents.copy;
begin
end;

procedure TWebContents.copyImageAt(X, Y: NativeInt);
begin
end;

procedure TWebContents.cut;
begin
end;

procedure TWebContents.delete;
begin
end;

procedure TWebContents.downloadURL(aURL: string);
begin
end;

procedure TWebContents.focus;
begin
end;

function TWebContents.getTitle: string;
begin
end;

function TWebContents.getURL: string;
begin
end;

procedure TWebContents.goBack;
begin
end;

procedure TWebContents.goForward;
begin
end;

function TWebContents.insertText(Text: string): TJSPromise;
begin
end;

function TWebContents.isDestroyed: Boolean;
begin
end;

function TWebContents.isFocused: Boolean;
begin
end;

function TWebContents.isLoading: Boolean;
begin
end;

function TWebContents.loadFile(aFilePath: string): TJSPromise;
begin
end;

function TWebContents.loadURL(aURL: string): TJSPromise;
begin
end;

procedure TWebContents.openDevTools;
begin
end;

procedure TWebContents.paste;
begin
end;

procedure TWebContents.redo;
begin
end;

procedure TWebContents.reload;
begin
end;

procedure TWebContents.replace(Text: string);
begin
end;

procedure TWebContents.replaceMisspelling(Text: string);
begin
end;

procedure TWebContents.selectAll;
begin
end;

procedure TWebContents.send(aChannel: string);
begin
end;

procedure TWebContents.send(aChannel: string; aArgs: JSValue);
begin
end;

procedure TWebContents.send(aChannel: string; aArgf, aArgs: JSValue);
begin
end;

procedure TWebContents.startDrag(aItem: TJSObject);
begin
end;

procedure TWebContents.stop;
begin
end;

procedure TWebContents.toggleDevTools;
begin
end;

procedure TWebContents.undo;
begin
end;

procedure TWebContents.unselect;
begin
end;

procedure TWebContents.&on(aChannel: string;
  aListener: TWebContentsEventArgsHandler);
begin
end;

procedure TWebContents.&on(aChannel: string;
  aListener: TWebContentsEventHandler);
begin
end;

{ TBrowserWindow }

procedure TBrowserWindow.blur;
begin
end;

procedure TBrowserWindow.close;
begin
end;

procedure TBrowserWindow.destroy;
begin
end;

procedure TBrowserWindow.focus;
begin
end;

function TBrowserWindow.fromId(aId: NativeInt): TBrowserWindow;
begin
end;

function TBrowserWindow.getAllWindows: TJSObjectDynArray;
begin
end;

function TBrowserWindow.getFocusedWindow: TBrowserWindow;
begin
end;

function TBrowserWindow.getMaximumSize: TJSArray;
begin
end;

function TBrowserWindow.getMinimumSize: TJSArray;
begin
end;

function TBrowserWindow.getParentWindow: TBrowserWindow;
begin
end;

function TBrowserWindow.getSize: TJSArray;
begin
end;

function TBrowserWindow.getTitle: string;
begin
end;

procedure TBrowserWindow.hide;
begin
end;

function TBrowserWindow.isAlwaysOnTop: Boolean;
begin
end;

function TBrowserWindow.isFocused: Boolean;
begin
end;

function TBrowserWindow.isFullScreen: Boolean;
begin
end;

function TBrowserWindow.isKiosk: Boolean;
begin
end;

function TBrowserWindow.isVisible: Boolean;
begin
end;

function TBrowserWindow.loadFile(aFilePath: string): TJSPromise;
begin
end;

function TBrowserWindow.loadURL(aURL: string): TJSPromise;
begin
end;

procedure TBrowserWindow.setAlwaysOnTop(aFlag: Boolean);
begin
end;

procedure TBrowserWindow.setContentSize(aWidth, aHeight: NativeInt);
begin
end;

procedure TBrowserWindow.setEnabled(aEnable: Boolean);
begin
end;

procedure TBrowserWindow.setFullScreen(aFlag: Boolean);
begin
end;

procedure TBrowserWindow.setKiosk(aFlag: Boolean);
begin
end;

procedure TBrowserWindow.setMaximumSize(aWidth, aHeight: NativeInt);
begin
end;

procedure TBrowserWindow.setMinimumSize(aWidth, aHeight: NativeInt);
begin
end;

procedure TBrowserWindow.setSize(aWidth, aHeight: NativeInt);
begin
end;

procedure TBrowserWindow.setSkipTaskbar(aSkip: Boolean);
begin
end;

procedure TBrowserWindow.setTitle(aTitle: string);
begin
end;

procedure TBrowserWindow.show;
begin
end;

{ TIPCMain }

procedure TIPCMain.once(aChannel: string; aListener: TIPCEventArgsHandler);
begin
end;

procedure TIPCMain.once(aChannel: string; aListener: TIPCEventHandler);
begin
end;

procedure TIPCMain.removeAllListeners;
begin
end;

procedure TIPCMain.removeAllListeners(aChannel: string);
begin
end;

procedure TIPCMain.removeListener(aChannel: string;
  aListener: TIPCEventHandler);
begin
end;

procedure TIPCMain.removeListener(aChannel: string;
  aListener: TIPCEventArgsHandler);
begin
end;

procedure TIPCMain.&on(aChannel: string; aListener: TIPCEventHandler);
begin
end;

procedure TIPCMain.&on(aChannel: string; aListener: TIPCEventArgsHandler);
begin
end;

{ TIPCRenderer }

procedure TIPCRenderer.once(aChannel: string; aListener: TIPCEventArgsHandler);
begin
end;

procedure TIPCRenderer.once(aChannel: string; aListener: TIPCEventHandler);
begin
end;

procedure TIPCRenderer.removeAllListeners(aChannel: string);
begin
end;

procedure TIPCRenderer.removeListener(aChannel: string;
  aListener: TIPCEventHandler);
begin
end;

procedure TIPCRenderer.removeListener(aChannel: string;
  aListener: TIPCEventArgsHandler);
begin
end;

procedure TIPCRenderer.send(aChannel: string; aArgs: JSValue);
begin
end;

procedure TIPCRenderer.send(aChannel: string; aArgf, aArgs: JSValue);
begin
end;

procedure TIPCRenderer.send(aChannel: string);
begin
end;

procedure TIPCRenderer.sendTo(aWebContentsID: NativeInt; aChannel: string);
begin
end;

procedure TIPCRenderer.sendTo(aWebContentsID: NativeInt; aChannel: string;
  aArgs: JSValue);
begin
end;

function TIPCRenderer.invoke(aChannel: string; aArgs: JSValue): TJSPromise;
begin
end;

function TIPCRenderer.invoke(aChannel: string): TJSPromise;
begin
end;

procedure TIPCRenderer.&on(aChannel: string; aListener: TIPCEventArgsHandler);
begin
end;

procedure TIPCRenderer.&on(aChannel: string; aListener: TIPCEventHandler);
begin
end;

{ TNativeImageClass }

function TNativeImageClass.toDataURL: string;
begin
end;

{ TNativeImage }

function TNativeImage.createEmpty: TNativeImageClass;
begin
end;

function TNativeImage.createFromDataURL(aDataURL: string): TNativeImageClass;
begin
end;

function TNativeImage.createFromPath(aPath: string): TNativeImageClass;
begin
end;

{ TClipboard }

function TClipboard.availableFormats: TJSElectronStringDynArray;
begin
end;

procedure TClipboard.clear;
begin
end;

function TClipboard.readHTML: string;
begin
end;

function TClipboard.readImage: TNativeImageClass;
begin
end;

function TClipboard.readRTF: string;
begin
end;

function TClipboard.readText: string;
begin
end;

procedure TClipboard.writeHTML(aMarkup: string);
begin
end;

procedure TClipboard.writeImage(aImage: TNativeImageClass);
begin
end;

procedure TClipboard.writeRTF(aText: string);
begin
end;

procedure TClipboard.writeText(aText: string);
begin
end;

{ TShell }

procedure TShell.beep;
begin
end;

function TShell.moveItemToTrash(aFullPath: string): Boolean;
begin
end;

function TShell.openExternal(aURL: string): Boolean;
begin
end;

function TShell.openItem(aFullPath: string): Boolean;
begin
end;

function TShell.showItemInFolder(aFullPath: string): Boolean;
begin
end;

{ TMenuItemClass }

procedure TMenuItemClass.addEventListener(aName: string; aListener: JSValue);
begin
end;

procedure TMenuItemClass.click(aMenuItem: TMenuItemClass;
  aBrowserWindow: TBrowserWindow; aEvent: TJSEvent);
begin
end;

{ TMenuClass }

procedure TMenuClass.append(aMenuItem: TMenuItemClass);
begin
end;

procedure TMenuClass.closePopup;
begin
end;

procedure TMenuClass.closePopup(aBrowserWindow: TBrowserWindow);
begin
end;

function TMenuClass.getMenuItemById(aId: string): TMenuItemClass;
begin
end;

procedure TMenuClass.popup(aOptions: TJSObject);
begin
end;

{ TMenu }

function TMenu.getApplicationMenu: TMenuClass;
begin
end;

procedure TMenu.setApplicationMenu(aMenu: TMenuClass);
begin
end;

{ TTrayIcon }

procedure TTrayIcon.destroy;
begin
end;

procedure TTrayIcon.setContextMenu(aMenu: TMenuClass);
begin
end;

procedure TTrayIcon.setImage(aImage: TNativeImageClass);
begin
end;

procedure TTrayIcon.setToolTip(aToolTip: string);
begin
end;

procedure TTrayIcon.&on(aChannel: string;
  aListener: TTrayIconClickEventHandler);
begin
end;

{ TApplication }

function TApplication.getAppPath: string;
begin
end;

function TApplication.getLocale: string;
begin
end;

function TApplication.getName: string;
begin
end;

function TApplication.getPath(aName: string): string;
begin
end;

function TApplication.isReady: Boolean;
begin
end;

procedure TApplication.quit;
begin
end;

{ TGlobalShortcut }

function TGlobalShortcut.isRegistered(aAccelerator: string): Boolean;
begin
end;

procedure TGlobalShortcut.unregister(aAccelerator: string);
begin
end;

procedure TGlobalShortcut.unregisterAll;
begin
end;

procedure TGlobalShortcut.&register(aAccelerator: string;
  aCallback: TGlobalShortcutCallback);
begin
end;

{ TApp }

function TApp.getCurrentWebContents: TWebContents;
begin
end;

function TApp.getCurrentWindow: TBrowserWindow;
begin
end;

{ TJSElectronFileList }

function TJSElectronFileList.item(aIndex: NativeInt): TJSElectronFile;
begin
end;

{ TNodeFS }

function TNodeFS.existSync(aPath: string): Boolean;
begin
end;

//procedure TNodeFS.readFile(aPath: string; aCallBack: TReadFileCallBack);
//begin
//end;

function TNodeFS.readFileSync(aPath: string): TJSObject;
begin
end;

//procedure TNodeFS.unlink(aPath: string; aCallback: TUnlinkCallBack);
//begin
//end;

procedure TNodeFS.unwatchFile(aFileName: string; aListener: TWatchFileListener);
begin
end;

procedure TNodeFS.watchFile(aFileName: string; aListener: TWatchFileListener);
begin
end;

//procedure TNodeFS.writeFile(aPath, aData: string;
//  aCallBack: TWriteFileCallBack);
//begin
//end;

procedure TNodeFS.writeFileSync(aPath, aData: string);
begin
end;

function Electron: TElectron;
begin
end;

function FS: TNodeFS;
begin
end;

{ TJSElectronNotification }

procedure TJSElectronNotification.close;
begin
end;

constructor TJSElectronNotification.new(aTitle: string; AOptions: TJSObject);
begin
end;

constructor TJSElectronNotification.new(aTitle: string);
begin
end;

class function TJSElectronNotification.requestPermission: TJSPromise;
begin
end;

class procedure TJSElectronNotification.requestPermission(
  ACallBack: TJSNotificationPermissionCallBack);
begin
end;

{ TDialog }

procedure TDialog.showErrorBox(aTitle, aContent: string);
begin
end;

function TDialog.showMessageBox(aBrowserWindow: TBrowserWindow;
  aOptions: TJSObject): TJSPromise;
begin
end;

function TDialog.showMessageBox(aOptions: TJSObject): TJSPromise;
begin
end;

function TDialog.showMessageBoxSync(aOptions: TJSObject): NativeInt;
begin
end;

function TDialog.showMessageBoxSync(aBrowserWindow: TBrowserWindow;
  aOptions: TJSObject): NativeInt;
begin
end;

function TDialog.showOpenDialog(aOptions: TJSObject): TJSPromise;
begin
end;

function TDialog.showOpenDialog(aBrowserWindow: TBrowserWindow;
  aOptions: TJSObject): TJSPromise;
begin
end;

function TDialog.showOpenDialogSync(
  aOptions: TJSObject): TJSElectronStringDynArray;
begin
end;

function TDialog.showOpenDialogSync(aBrowserWindow: TBrowserWindow;
  aOptions: TJSObject): TJSElectronStringDynArray;
begin
end;

function TDialog.showSaveDialog(aOptions: TJSObject): TJSPromise;
begin
end;

function TDialog.showSaveDialog(aBrowserWindow: TBrowserWindow;
  aOptions: TJSObject): TJSPromise;
begin
end;

function TDialog.showSaveDialogSync(aOptions: TJSObject): string;
begin
end;

function TDialog.showSaveDialogSync(aBrowserWindow: TBrowserWindow;
  aOptions: TJSObject): string;
begin
end;

end.
